# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "policy set-definition list",
)
class List(AAZCommand):
    """Retrieve policy set definitions.

    Retrieve the list of all policy set definitions in the given subscription or management group.

    :example: List policy set definitions
        az policy set-definition list

    :example: List policy set definitions at management group level
        az policy set-definition list --management-group MyManagementGroup
    """

    _aaz_info = {
        "version": "2024-05-01",
        "resources": [
            ["mgmt-plane", "/providers/microsoft.authorization/policysetdefinitions", "2024-05-01"],
            ["mgmt-plane", "/providers/microsoft.management/managementgroups/{}/providers/microsoft.authorization/policysetdefinitions", "2024-05-01"],
            ["mgmt-plane", "/subscriptions/{}/providers/microsoft.authorization/policysetdefinitions", "2024-05-01"],
        ]
    }

    AZ_SUPPORT_PAGINATION = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_paging(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.management_group = AAZStrArg(
            options=["--management-group"],
            help={"short-summary": "The management group.", "long-summary": "The management group with the given name whose policy set definitions are to be listed."},
        )
        _args_schema.expand = AAZStrArg(
            options=["--expand"],
            help={"short-summary": "Additional properties to include in output", "long-summary": "Comma-separated list of additional properties to include in the command output. Supported values are 'LatestDefinitionVersion, EffectiveDefinitionVersion'."},
        )
        _args_schema.filter = AAZStrArg(
            options=["--filter"],
            help={"short-summary": "Filter list results.", "long-summary": "The filter to limit list results. Valid values are: 'atScope()', 'atExactScope()', 'atScopeAndBelow()' or 'policyDefinitionId eq '{value}''. If filter is not provided, no filtering is performed. If filter atScope() is provided, the returned list includes all policy assignments that apply to the given scope, which is everything in the unfiltered list except those applied to sub scopes contained within the given scope. If filter atExactScope() is provided, the returned list includes all policy assignments at the given scope.  If filter atScopeAndBelow() is provided, the returned list includes all policy assignments at the given scope and those in sub scopes contained within the given scope. If filter policyDefinitionId eq '{value}' is provided, the returned list includes all policy assignments of the policy definition whose id is {value}."},
        )
        return cls._args_schema

    def _execute_operations(self):
        self.pre_operations()
        condition_0 = has_value(self.ctx.args.management_group) is not True and has_value(self.ctx.subscription_id) is not True
        condition_1 = has_value(self.ctx.args.management_group)
        condition_2 = has_value(self.ctx.subscription_id)
        if condition_0:
            self.PolicySetDefinitionsListBuiltIn(ctx=self.ctx)()
        if condition_1:
            self.PolicySetDefinitionsListByManagementGroup(ctx=self.ctx)()
        if condition_2:
            self.PolicySetDefinitionsList(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance.value, client_flatten=True)
        next_link = self.deserialize_output(self.ctx.vars.instance.next_link)
        return result, next_link

    class PolicySetDefinitionsListBuiltIn(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/providers/Microsoft.Authorization/policySetDefinitions",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "$expand", self.ctx.args.expand,
                ),
                **self.serialize_query_param(
                    "$filter", self.ctx.args.filter,
                ),
                **self.serialize_query_param(
                    "api-version", "2024-05-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.next_link = AAZStrType(
                serialized_name="nextLink",
            )
            _schema_on_200.value = AAZListType()

            value = cls._schema_on_200.value
            value.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element
            _element.id = AAZStrType(
                flags={"read_only": True},
            )
            _element.name = AAZStrType(
                flags={"read_only": True},
            )
            _element.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _element.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _element.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.value.Element.properties
            properties.description = AAZStrType()
            properties.display_name = AAZStrType(
                serialized_name="displayName",
            )
            properties.metadata = AAZDictType()
            properties.parameters = AAZDictType()
            properties.policy_definition_groups = AAZListType(
                serialized_name="policyDefinitionGroups",
            )
            properties.policy_definitions = AAZListType(
                serialized_name="policyDefinitions",
                flags={"required": True},
            )
            properties.policy_type = AAZStrType(
                serialized_name="policyType",
            )
            properties.version = AAZStrType()
            properties.versions = AAZListType()

            metadata = cls._schema_on_200.value.Element.properties.metadata
            metadata.Element = AAZAnyType()

            parameters = cls._schema_on_200.value.Element.properties.parameters
            parameters.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.parameters.Element
            _element.allowed_values = AAZListType(
                serialized_name="allowedValues",
            )
            _element.default_value = AAZAnyType(
                serialized_name="defaultValue",
            )
            _element.metadata = AAZFreeFormDictType()
            _element.schema = AAZDictType()
            _element.type = AAZStrType()

            allowed_values = cls._schema_on_200.value.Element.properties.parameters.Element.allowed_values
            allowed_values.Element = AAZAnyType()

            schema = cls._schema_on_200.value.Element.properties.parameters.Element.schema
            schema.Element = AAZAnyType()

            policy_definition_groups = cls._schema_on_200.value.Element.properties.policy_definition_groups
            policy_definition_groups.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.policy_definition_groups.Element
            _element.additional_metadata_id = AAZStrType(
                serialized_name="additionalMetadataId",
            )
            _element.category = AAZStrType()
            _element.description = AAZStrType()
            _element.display_name = AAZStrType(
                serialized_name="displayName",
            )
            _element.name = AAZStrType(
                flags={"required": True},
            )

            policy_definitions = cls._schema_on_200.value.Element.properties.policy_definitions
            policy_definitions.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.policy_definitions.Element
            _element.definition_version = AAZStrType(
                serialized_name="definitionVersion",
            )
            _element.effective_definition_version = AAZStrType(
                serialized_name="effectiveDefinitionVersion",
                flags={"read_only": True},
            )
            _element.group_names = AAZListType(
                serialized_name="groupNames",
            )
            _element.latest_definition_version = AAZStrType(
                serialized_name="latestDefinitionVersion",
                flags={"read_only": True},
            )
            _element.parameters = AAZDictType()
            _element.policy_definition_id = AAZStrType(
                serialized_name="policyDefinitionId",
                flags={"required": True},
            )
            _element.policy_definition_reference_id = AAZStrType(
                serialized_name="policyDefinitionReferenceId",
            )

            group_names = cls._schema_on_200.value.Element.properties.policy_definitions.Element.group_names
            group_names.Element = AAZStrType()

            parameters = cls._schema_on_200.value.Element.properties.policy_definitions.Element.parameters
            parameters.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.policy_definitions.Element.parameters.Element
            _element.value = AAZAnyType()

            versions = cls._schema_on_200.value.Element.properties.versions
            versions.Element = AAZStrType()

            system_data = cls._schema_on_200.value.Element.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            return cls._schema_on_200

    class PolicySetDefinitionsListByManagementGroup(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/providers/Microsoft.Management/managementGroups/{managementGroupId}/providers/Microsoft.Authorization/policySetDefinitions",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "managementGroupId", self.ctx.args.management_group,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "$expand", self.ctx.args.expand,
                ),
                **self.serialize_query_param(
                    "$filter", self.ctx.args.filter,
                ),
                **self.serialize_query_param(
                    "api-version", "2024-05-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.next_link = AAZStrType(
                serialized_name="nextLink",
            )
            _schema_on_200.value = AAZListType()

            value = cls._schema_on_200.value
            value.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element
            _element.id = AAZStrType(
                flags={"read_only": True},
            )
            _element.name = AAZStrType(
                flags={"read_only": True},
            )
            _element.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _element.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _element.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.value.Element.properties
            properties.description = AAZStrType()
            properties.display_name = AAZStrType(
                serialized_name="displayName",
            )
            properties.metadata = AAZDictType()
            properties.parameters = AAZDictType()
            properties.policy_definition_groups = AAZListType(
                serialized_name="policyDefinitionGroups",
            )
            properties.policy_definitions = AAZListType(
                serialized_name="policyDefinitions",
                flags={"required": True},
            )
            properties.policy_type = AAZStrType(
                serialized_name="policyType",
            )
            properties.version = AAZStrType()
            properties.versions = AAZListType()

            metadata = cls._schema_on_200.value.Element.properties.metadata
            metadata.Element = AAZAnyType()

            parameters = cls._schema_on_200.value.Element.properties.parameters
            parameters.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.parameters.Element
            _element.allowed_values = AAZListType(
                serialized_name="allowedValues",
            )
            _element.default_value = AAZAnyType(
                serialized_name="defaultValue",
            )
            _element.metadata = AAZFreeFormDictType()
            _element.schema = AAZDictType()
            _element.type = AAZStrType()

            allowed_values = cls._schema_on_200.value.Element.properties.parameters.Element.allowed_values
            allowed_values.Element = AAZAnyType()

            schema = cls._schema_on_200.value.Element.properties.parameters.Element.schema
            schema.Element = AAZAnyType()

            policy_definition_groups = cls._schema_on_200.value.Element.properties.policy_definition_groups
            policy_definition_groups.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.policy_definition_groups.Element
            _element.additional_metadata_id = AAZStrType(
                serialized_name="additionalMetadataId",
            )
            _element.category = AAZStrType()
            _element.description = AAZStrType()
            _element.display_name = AAZStrType(
                serialized_name="displayName",
            )
            _element.name = AAZStrType(
                flags={"required": True},
            )

            policy_definitions = cls._schema_on_200.value.Element.properties.policy_definitions
            policy_definitions.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.policy_definitions.Element
            _element.definition_version = AAZStrType(
                serialized_name="definitionVersion",
            )
            _element.effective_definition_version = AAZStrType(
                serialized_name="effectiveDefinitionVersion",
                flags={"read_only": True},
            )
            _element.group_names = AAZListType(
                serialized_name="groupNames",
            )
            _element.latest_definition_version = AAZStrType(
                serialized_name="latestDefinitionVersion",
                flags={"read_only": True},
            )
            _element.parameters = AAZDictType()
            _element.policy_definition_id = AAZStrType(
                serialized_name="policyDefinitionId",
                flags={"required": True},
            )
            _element.policy_definition_reference_id = AAZStrType(
                serialized_name="policyDefinitionReferenceId",
            )

            group_names = cls._schema_on_200.value.Element.properties.policy_definitions.Element.group_names
            group_names.Element = AAZStrType()

            parameters = cls._schema_on_200.value.Element.properties.policy_definitions.Element.parameters
            parameters.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.policy_definitions.Element.parameters.Element
            _element.value = AAZAnyType()

            versions = cls._schema_on_200.value.Element.properties.versions
            versions.Element = AAZStrType()

            system_data = cls._schema_on_200.value.Element.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            return cls._schema_on_200

    class PolicySetDefinitionsList(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/providers/Microsoft.Authorization/policySetDefinitions",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "MgmtErrorFormat"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "$expand", self.ctx.args.expand,
                ),
                **self.serialize_query_param(
                    "$filter", self.ctx.args.filter,
                ),
                **self.serialize_query_param(
                    "api-version", "2024-05-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()

            _schema_on_200 = cls._schema_on_200
            _schema_on_200.next_link = AAZStrType(
                serialized_name="nextLink",
            )
            _schema_on_200.value = AAZListType()

            value = cls._schema_on_200.value
            value.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element
            _element.id = AAZStrType(
                flags={"read_only": True},
            )
            _element.name = AAZStrType(
                flags={"read_only": True},
            )
            _element.properties = AAZObjectType(
                flags={"client_flatten": True},
            )
            _element.system_data = AAZObjectType(
                serialized_name="systemData",
                flags={"read_only": True},
            )
            _element.type = AAZStrType(
                flags={"read_only": True},
            )

            properties = cls._schema_on_200.value.Element.properties
            properties.description = AAZStrType()
            properties.display_name = AAZStrType(
                serialized_name="displayName",
            )
            properties.metadata = AAZDictType()
            properties.parameters = AAZDictType()
            properties.policy_definition_groups = AAZListType(
                serialized_name="policyDefinitionGroups",
            )
            properties.policy_definitions = AAZListType(
                serialized_name="policyDefinitions",
                flags={"required": True},
            )
            properties.policy_type = AAZStrType(
                serialized_name="policyType",
            )
            properties.version = AAZStrType()
            properties.versions = AAZListType()

            metadata = cls._schema_on_200.value.Element.properties.metadata
            metadata.Element = AAZAnyType()

            parameters = cls._schema_on_200.value.Element.properties.parameters
            parameters.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.parameters.Element
            _element.allowed_values = AAZListType(
                serialized_name="allowedValues",
            )
            _element.default_value = AAZAnyType(
                serialized_name="defaultValue",
            )
            _element.metadata = AAZFreeFormDictType()
            _element.schema = AAZDictType()
            _element.type = AAZStrType()

            allowed_values = cls._schema_on_200.value.Element.properties.parameters.Element.allowed_values
            allowed_values.Element = AAZAnyType()

            schema = cls._schema_on_200.value.Element.properties.parameters.Element.schema
            schema.Element = AAZAnyType()

            policy_definition_groups = cls._schema_on_200.value.Element.properties.policy_definition_groups
            policy_definition_groups.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.policy_definition_groups.Element
            _element.additional_metadata_id = AAZStrType(
                serialized_name="additionalMetadataId",
            )
            _element.category = AAZStrType()
            _element.description = AAZStrType()
            _element.display_name = AAZStrType(
                serialized_name="displayName",
            )
            _element.name = AAZStrType(
                flags={"required": True},
            )

            policy_definitions = cls._schema_on_200.value.Element.properties.policy_definitions
            policy_definitions.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.policy_definitions.Element
            _element.definition_version = AAZStrType(
                serialized_name="definitionVersion",
            )
            _element.effective_definition_version = AAZStrType(
                serialized_name="effectiveDefinitionVersion",
                flags={"read_only": True},
            )
            _element.group_names = AAZListType(
                serialized_name="groupNames",
            )
            _element.latest_definition_version = AAZStrType(
                serialized_name="latestDefinitionVersion",
                flags={"read_only": True},
            )
            _element.parameters = AAZDictType()
            _element.policy_definition_id = AAZStrType(
                serialized_name="policyDefinitionId",
                flags={"required": True},
            )
            _element.policy_definition_reference_id = AAZStrType(
                serialized_name="policyDefinitionReferenceId",
            )

            group_names = cls._schema_on_200.value.Element.properties.policy_definitions.Element.group_names
            group_names.Element = AAZStrType()

            parameters = cls._schema_on_200.value.Element.properties.policy_definitions.Element.parameters
            parameters.Element = AAZObjectType()

            _element = cls._schema_on_200.value.Element.properties.policy_definitions.Element.parameters.Element
            _element.value = AAZAnyType()

            versions = cls._schema_on_200.value.Element.properties.versions
            versions.Element = AAZStrType()

            system_data = cls._schema_on_200.value.Element.system_data
            system_data.created_at = AAZStrType(
                serialized_name="createdAt",
            )
            system_data.created_by = AAZStrType(
                serialized_name="createdBy",
            )
            system_data.created_by_type = AAZStrType(
                serialized_name="createdByType",
            )
            system_data.last_modified_at = AAZStrType(
                serialized_name="lastModifiedAt",
            )
            system_data.last_modified_by = AAZStrType(
                serialized_name="lastModifiedBy",
            )
            system_data.last_modified_by_type = AAZStrType(
                serialized_name="lastModifiedByType",
            )

            return cls._schema_on_200


class _ListHelper:
    """Helper class for List"""


__all__ = ["List"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from typing import Dict

from azure.core.polling.base_polling import LocationPolling, _is_empty, BadResponse, _as_json


class RunCommandLocationPolling(LocationPolling):
    """Extends LocationPolling but uses the body content instead of the status code for the status"""

    @staticmethod
    def _get_provisioning_state(response):
        """Attempt to get provisioning state from resource.

        :param azure.core.pipeline.transport.HttpResponse response: latest REST call response.
        :returns: Status if found, else 'None'.
        """
        if _is_empty(response):
            return None
        body: Dict = _as_json(response)
        return body.get("properties", {}).get("provisioningState")

    def get_status(self, pipeline_response):
        """Process the latest status update retrieved from the same URL as
        the previous request.

        :param azure.core.pipeline.PipelineResponse response: latest REST call response.
        :raises: BadResponse if status not 200 or 204.
        """
        response = pipeline_response.http_response
        if _is_empty(response):
            raise BadResponse(
                "The response from long running operation does not contain a body."
            )

        status = self._get_provisioning_state(response)
        return status or "Unknown"



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from .custom import default_content_types     # pylint: disable=unused-import
from .custom import list_profiles             # pylint: disable=unused-import
from .custom import get_profile               # pylint: disable=unused-import
from .custom import delete_profile            # pylint: disable=unused-import
from .custom import update_endpoint           # pylint: disable=unused-import
from .custom import create_condition          # pylint: disable=unused-import
from .custom import create_action             # pylint: disable=unused-import
from .custom import add_rule                  # pylint: disable=unused-import
from .custom import add_condition             # pylint: disable=unused-import
from .custom import add_action                # pylint: disable=unused-import
from .custom import remove_rule               # pylint: disable=unused-import
from .custom import remove_condition          # pylint: disable=unused-import
from .custom import remove_action             # pylint: disable=unused-import
from .custom import create_endpoint           # pylint: disable=unused-import
from .custom import purge_endpoint_content    # pylint: disable=unused-import
from .custom import load_endpoint_content     # pylint: disable=unused-import
from .custom import create_custom_domain      # pylint: disable=unused-import
from .custom import enable_custom_https       # pylint: disable=unused-import
from .custom import update_origin             # pylint: disable=unused-import
from .custom import create_origin             # pylint: disable=unused-import
from .custom import update_profile            # pylint: disable=unused-import
from .custom import create_profile            # pylint: disable=unused-import
from .custom import create_origin_group       # pylint: disable=unused-import
from .custom import update_origin_group       # pylint: disable=unused-import
from .custom import check_name_availability   # pylint: disable=unused-import
from .custom import validate_custom_domain    # pylint: disable=unused-import



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------


_ENV_AZ_INSTALLER = 'AZ_INSTALLER'
_ENV_AZ_BICEP_GLOBALIZATION_INVARIANT = 'AZ_BICEP_GLOBALIZATION_INVARIANT'


def get_config_dir():
    import os
    return os.getenv('AZURE_CONFIG_DIR', None) or os.path.expanduser(os.path.join('~', '.azure'))



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

"""Assist the command module to get correct type from SDK based on current API version"""

from azure.cli.core.profiles import get_sdk, ResourceType


def get_blob_types():
    return 'block', 'page', 'append'


def get_blob_tier_names_track2(cli_ctx, model_path):
    t_blob_tier_model = get_sdk(cli_ctx, ResourceType.DATA_STORAGE_BLOB, model_path)
    return [v for v in dir(t_blob_tier_model) if not v.startswith('_')]


def get_delete_blob_snapshot_type_names():
    return 'include', 'only'


def get_container_access_type_names():
    return 'off', 'blob', 'container'


def get_fs_access_type_names():
    return 'off', 'file', 'filesystem'


def get_fs_access_type(cli_ctx, name):
    if name == 'off':
        return None
    if name == 'file':
        return get_sdk(cli_ctx, ResourceType.DATA_STORAGE_FILEDATALAKE, 'PublicAccess', mod='_models').File
    if name == 'filesystem':
        return get_sdk(cli_ctx, ResourceType.DATA_STORAGE_FILEDATALAKE, 'PublicAccess', mod='_models').FileSystem
    raise KeyError


def get_blob_sync_delete_destination_types():
    return 'true', 'false', 'prompt'



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# vmss application health setting for rolling upgrade policy

application_health_setting_for_linux = '''
#cloud-config
package_upgrade: true

packages:
  - nginx

write_files:
  - path: /var/www/html/index.html
    content: |
      <!DOCTYPE html>
      <html>
      <body>
      <h1>Hello, World!</h1>
      </body>
      </html>
    owner: www-data:www-data
    permissions: '0644'
  - path: /var/www/html/health.html
    content: |
      healthy
    owner: www-data:www-data
    permissions: '0644'
  - path: /etc/nginx/sites-available/default
    content: |
      server {
          listen 80 default_server;
          listen [::]:80 default_server;
          root /var/www/html;
          index index.html;
          server_name _;
          location / {
              try_files $uri $uri/ =404;
          }
          location /health {
              try_files $uri.html =404;
          }
      }
    owner: root:root
    permissions: '0644'

runcmd:
  - systemctl enable nginx
  - service nginx restart
'''

application_health_setting_for_windows = '''
#!/bin/bash

apt-get update -y && apt-get upgrade -y
apt-get install -y nginx
echo "Hello World from host" $HOSTNAME "!" | sudo tee -a /var/www/html/index.html
'''



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from collections.abc import Mapping
from azure.core.paging import ItemPaged


class Transformer:

    def __init__(self, transform_mapping):
        self.transform_mapping = transform_mapping

    def transform_result(self, result):
        if result is not None:
            result = list(result) if isinstance(result, ItemPaged) else result
            if isinstance(result, list):
                return self.transform_object_list(result)
            return self.transform_object(result)

    def transform_object(self, result):
        new_dict = {}
        for key, value in result.items():
            new_key = self.transform_mapping[key] if key in self.transform_mapping else key
            if isinstance(value, Mapping):
                new_dict[new_key] = self.transform_result(value)
            else:
                new_dict[new_key] = value
        return new_dict

    def transform_object_list(self, result):
        new_result = []
        for item in result:
            updated_obj = self.transform_result(item)
            new_result.append(updated_obj)
        return new_result


transform_map = {
    # cmd: az batch pool list
    # cmd: az batch pool show
    'diskSizeGB': 'diskSizeGb',
    'ephemeralOSDiskSettings': 'ephemeralOsDiskSettings',
    'nodeAgentSKUId': 'nodeAgentSkuId',
    # cmd: az batch pool list
    'automaticOSUpgradePolicy': 'automaticOsUpgradePolicy',
    'dynamicVNetAssignmentScope': 'dynamicVnetAssignmentScope',
    'enableAutomaticOSUpgrade': 'enableAutomaticOsUpgrade',
    'publicIPAddressConfiguration': 'publicIpAddressConfiguration',
    # cmd: az batch node remote-login-settings show
    'remoteLoginIPAddress': 'remoteLoginIpAddress',
    # cmd:
    'inboundNATPools': 'inboundNatPools',
    'publicFQDN': 'publicFqdn',
    'publicIPAddress': 'publicIpAddress',
    'upgradingOS': 'upgradingOs',
    # cmd: az batch pool usage-metrics list (API was retired on 09/30/2024 so these should be removed soon)
    'avgCPUPercentage': 'avgCpuPercentage',
    'diskReadIOps': 'diskReadIops',
    'diskWriteIOps': 'diskWriteIops',
    'kernelCPUTime': 'kernelCpuTime',
    'readIOGiB': 'readIoGiB',
    'readIOps': 'readIops',
    'userCPUTime': 'userCpuTime',
    'writeIOGiB': 'writeIoGiB',
    'writeIOps': 'writeIops',
}

batch_transformer = Transformer(transform_map)



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from enum import Enum
from knack.log import get_logger
from knack.util import CLIError

from azure.cli.core.azclierror import RequiredArgumentMissingError
from azure.cli.core.commands import CliCommandType
from azure.cli.core.commands.client_factory import prepare_client_kwargs_track2
from azure.cli.core.profiles import get_api_version, ResourceType
from azure.cli.core._profile import Profile

logger = get_logger(__name__)


class Clients(str, Enum):
    vaults = 'vaults'
    private_endpoint_connections = 'private_endpoint_connections'
    private_link_resources = 'private_link_resources'
    managed_hsms = 'managed_hsms'
    mhsm_private_endpoint_connections = 'mhsm_private_endpoint_connections'
    mhsm_private_link_resources = 'mhsm_private_link_resources'
    mhsm_regions = 'mhsm_regions'
    private_7_2 = 'private_7_2'


OPERATIONS_NAME = {
    Clients.vaults: 'VaultsOperations',
    Clients.private_endpoint_connections: 'PrivateEndpointConnectionsOperations',
    Clients.private_link_resources: 'PrivateLinkResourcesOperations',
    Clients.managed_hsms: 'ManagedHsmsOperations',
    Clients.mhsm_private_endpoint_connections: 'MHSMPrivateEndpointConnectionsOperations',
    Clients.mhsm_private_link_resources: 'MHSMPrivateLinkResourcesOperations',
    Clients.mhsm_regions: 'MHSMRegionsOperations'
}

KEYVAULT_TEMPLATE_STRINGS = {
    ResourceType.MGMT_KEYVAULT:
        'azure.mgmt.keyvault.{module_name}#{class_name}{obj_name}',
    ResourceType.DATA_KEYVAULT_ADMINISTRATION_BACKUP:
        'azure.keyvault.administration._backup_client#KeyVaultBackupClient{obj_name}',
    ResourceType.DATA_KEYVAULT_ADMINISTRATION_ACCESS_CONTROL:
        'azure.keyvault.administration._access_control_client#KeyVaultAccessControlClient{obj_name}',
    ResourceType.DATA_KEYVAULT_ADMINISTRATION_SETTING:
        'azure.keyvault.administration._settings_client#KeyVaultSettingsClient{obj_name}',
    ResourceType.DATA_KEYVAULT_CERTIFICATES:
        'azure.keyvault.certificates._client#CertificateClient{obj_name}',
    ResourceType.DATA_KEYVAULT_KEYS:
        'azure.keyvault.keys._client#KeyClient{obj_name}',
    ResourceType.DATA_KEYVAULT_SECRETS:
        'azure.keyvault.secrets._client#SecretClient{obj_name}',
    ResourceType.DATA_KEYVAULT_SECURITY_DOMAIN:
        'azure.keyvault.securitydomain._client#SecurityDomainClient{obj_name}',
}


def is_mgmt_plane(resource_type):
    return resource_type == ResourceType.MGMT_KEYVAULT


def get_operations_tmpl(resource_type, client_name):
    if resource_type in [ResourceType.DATA_KEYVAULT_ADMINISTRATION_BACKUP,
                         ResourceType.DATA_KEYVAULT_ADMINISTRATION_ACCESS_CONTROL,
                         ResourceType.DATA_KEYVAULT_CERTIFICATES,
                         ResourceType.DATA_KEYVAULT_KEYS,
                         ResourceType.DATA_KEYVAULT_SECRETS,
                         ResourceType.DATA_KEYVAULT_SECURITY_DOMAIN,
                         ResourceType.DATA_KEYVAULT_ADMINISTRATION_SETTING]:
        return KEYVAULT_TEMPLATE_STRINGS[resource_type].format(obj_name='.{}')

    class_name = OPERATIONS_NAME.get(client_name, '') if is_mgmt_plane(resource_type) else 'KeyVaultClient'
    return KEYVAULT_TEMPLATE_STRINGS[resource_type].format(
        module_name='operations',
        class_name=class_name,
        obj_name='.{}')


def get_docs_tmpl(resource_type, client_name, module_name='operations'):
    if resource_type in [ResourceType.DATA_KEYVAULT_ADMINISTRATION_BACKUP,
                         ResourceType.DATA_KEYVAULT_ADMINISTRATION_ACCESS_CONTROL,
                         ResourceType.DATA_KEYVAULT_CERTIFICATES,
                         ResourceType.DATA_KEYVAULT_KEYS,
                         ResourceType.DATA_KEYVAULT_SECRETS,
                         ResourceType.DATA_KEYVAULT_SECURITY_DOMAIN,
                         ResourceType.DATA_KEYVAULT_ADMINISTRATION_SETTING]:
        return KEYVAULT_TEMPLATE_STRINGS[resource_type].format(obj_name='.{}')

    if is_mgmt_plane(resource_type):
        class_name = OPERATIONS_NAME.get(client_name, '') + '.' if module_name == 'operations' else ''
    else:
        class_name = 'KeyVaultClient.'
    return KEYVAULT_TEMPLATE_STRINGS[resource_type].format(
        module_name=module_name,
        class_name=class_name,
        obj_name='{}')


# pylint: disable=too-many-return-statements
def get_client_factory(resource_type, client_name=''):
    if is_mgmt_plane(resource_type):
        return keyvault_mgmt_client_factory(resource_type, client_name)
    if resource_type == ResourceType.DATA_KEYVAULT_ADMINISTRATION_BACKUP:
        return data_plane_azure_keyvault_administration_backup_client
    if resource_type == ResourceType.DATA_KEYVAULT_ADMINISTRATION_ACCESS_CONTROL:
        return data_plane_azure_keyvault_administration_access_control_client
    if resource_type == ResourceType.DATA_KEYVAULT_ADMINISTRATION_SETTING:
        return data_plane_azure_keyvault_administration_setting_client
    if resource_type == ResourceType.DATA_KEYVAULT_CERTIFICATES:
        return data_plane_azure_keyvault_certificate_client
    if resource_type == ResourceType.DATA_KEYVAULT_KEYS:
        return data_plane_azure_keyvault_key_client
    if resource_type == ResourceType.DATA_KEYVAULT_SECRETS:
        return data_plane_azure_keyvault_secret_client
    if resource_type == ResourceType.DATA_KEYVAULT_SECURITY_DOMAIN:
        return data_plane_azure_keyvault_security_domain_client
    raise CLIError('Unsupported resource type: {}'.format(resource_type))


class ClientEntity:  # pylint: disable=too-few-public-methods
    def __init__(self, client_factory, command_type, operations_docs_tmpl, models_docs_tmpl):
        self.client_factory = client_factory
        self.command_type = command_type
        self.operations_docs_tmpl = operations_docs_tmpl
        self.models_docs_tmpl = models_docs_tmpl


def get_client(resource_type, client_name=''):
    client_factory = get_client_factory(resource_type, client_name)
    command_type = CliCommandType(
        operations_tmpl=get_operations_tmpl(resource_type, client_name),
        client_factory=client_factory,
        resource_type=resource_type
    )
    operations_docs_tmpl = get_docs_tmpl(resource_type, client_name, module_name='operations')
    models_docs_tmpl = get_docs_tmpl(resource_type, client_name, module_name='models')
    return ClientEntity(client_factory, command_type, operations_docs_tmpl, models_docs_tmpl)


def keyvault_mgmt_client_factory(resource_type, client_name):
    def _keyvault_mgmt_client_factory(cli_ctx, _):
        from azure.cli.core.commands.client_factory import get_mgmt_service_client
        return getattr(get_mgmt_service_client(cli_ctx, resource_type), client_name)

    return _keyvault_mgmt_client_factory


def data_plane_azure_keyvault_administration_backup_client(cli_ctx, command_args):
    from azure.keyvault.administration import KeyVaultBackupClient

    vault_url, credential, version = _prepare_data_plane_azure_keyvault_client(
        cli_ctx, command_args, ResourceType.DATA_KEYVAULT_ADMINISTRATION_BACKUP)
    client_kwargs = prepare_client_kwargs_track2(cli_ctx)
    client_kwargs.pop('http_logging_policy')
    return KeyVaultBackupClient(
        vault_url=vault_url, credential=credential, api_version=version,
        verify_challenge_resource=False, **client_kwargs)


def data_plane_azure_keyvault_administration_access_control_client(cli_ctx, command_args):
    from azure.keyvault.administration import KeyVaultAccessControlClient

    vault_url, credential, version = _prepare_data_plane_azure_keyvault_client(
        cli_ctx, command_args, ResourceType.DATA_KEYVAULT_ADMINISTRATION_ACCESS_CONTROL)
    client_kwargs = prepare_client_kwargs_track2(cli_ctx)
    client_kwargs.pop('http_logging_policy')
    return KeyVaultAccessControlClient(
        vault_url=vault_url, credential=credential, api_version=version,
        verify_challenge_resource=False, **client_kwargs)


def data_plane_azure_keyvault_administration_setting_client(cli_ctx, command_args):
    from azure.keyvault.administration import KeyVaultSettingsClient

    vault_url, credential, _ = _prepare_data_plane_azure_keyvault_client(
        cli_ctx, command_args, ResourceType.DATA_KEYVAULT_ADMINISTRATION_SETTING)
    command_args.pop('hsm_name', None)
    command_args.pop('vault_base_url', None)
    command_args.pop('identifier', None)
    client_kwargs = prepare_client_kwargs_track2(cli_ctx)
    client_kwargs.pop('http_logging_policy')
    return KeyVaultSettingsClient(
        vault_url=vault_url, credential=credential, api_version='7.4',
        verify_challenge_resource=False, **client_kwargs)


def data_plane_azure_keyvault_certificate_client(cli_ctx, command_args):
    from azure.keyvault.certificates import CertificateClient

    vault_url, credential, _ = _prepare_data_plane_azure_keyvault_client(
        cli_ctx, command_args, ResourceType.DATA_KEYVAULT_CERTIFICATES)
    command_args.pop('hsm_name', None)
    command_args.pop('vault_base_url', None)
    command_args.pop('identifier', None)
    client_kwargs = prepare_client_kwargs_track2(cli_ctx)
    client_kwargs.pop('http_logging_policy')
    return CertificateClient(
        vault_url=vault_url, credential=credential, api_version='7.4',
        verify_challenge_resource=False, **client_kwargs)


def data_plane_azure_keyvault_key_client(cli_ctx, command_args):
    from azure.keyvault.keys import KeyClient

    vault_url, credential, _ = _prepare_data_plane_azure_keyvault_client(
        cli_ctx, command_args, ResourceType.DATA_KEYVAULT_KEYS)
    command_args.pop('hsm_name', None)
    command_args.pop('vault_base_url', None)
    command_args.pop('identifier', None)
    client_kwargs = prepare_client_kwargs_track2(cli_ctx)
    client_kwargs.pop('http_logging_policy')
    return KeyClient(
        vault_url=vault_url, credential=credential, api_version='7.6-preview.2',
        verify_challenge_resource=False, **client_kwargs)


def data_plane_azure_keyvault_secret_client(cli_ctx, command_args):
    from azure.keyvault.secrets import SecretClient

    vault_url, credential, _ = _prepare_data_plane_azure_keyvault_client(
        cli_ctx, command_args, ResourceType.DATA_KEYVAULT_SECRETS)
    command_args.pop('hsm_name', None)
    command_args.pop('vault_base_url', None)
    command_args.pop('identifier', None)
    client_kwargs = prepare_client_kwargs_track2(cli_ctx)
    client_kwargs.pop('http_logging_policy')
    return SecretClient(
        vault_url=vault_url, credential=credential, api_version='7.4',
        verify_challenge_resource=False, **client_kwargs)


def data_plane_azure_keyvault_security_domain_client(cli_ctx, command_args):
    from azure.keyvault.securitydomain import SecurityDomainClient
    vault_url, credential, _ = _prepare_data_plane_azure_keyvault_client(
        cli_ctx, command_args, ResourceType.DATA_KEYVAULT_SECURITY_DOMAIN)
    command_args.pop('hsm_name', None)
    command_args.pop('vault_base_url', None)
    command_args.pop('identifier', None)
    client_kwargs = prepare_client_kwargs_track2(cli_ctx)
    client_kwargs.pop('http_logging_policy')
    return SecurityDomainClient(vault_url=vault_url, credential=credential,
                                verify_challenge_resource=False, **client_kwargs)


def _prepare_data_plane_azure_keyvault_client(cli_ctx, command_args, resource_type):
    version = str(get_api_version(cli_ctx, resource_type))
    profile = Profile(cli_ctx=cli_ctx)
    credential, _, _ = profile.get_login_credentials(subscription_id=cli_ctx.data.get('subscription_id'))
    vault_url = \
        command_args.get('hsm_name', None) or \
        command_args.get('vault_base_url', None) or \
        command_args.get('identifier', None)
    if not vault_url:
        raise RequiredArgumentMissingError('Please specify --hsm-name or --id')
    return vault_url, credential, version



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=line-too-long

from azure.mgmt.signalr.models import (
    ResourceSku,
    Replica)

from azure.mgmt.signalr.operations import SignalRReplicasOperations


def signalr_replica_list(client: SignalRReplicasOperations, resource_group_name, signalr_name):
    return client.list(resource_group_name, signalr_name)


def signalr_replica_show(client: SignalRReplicasOperations, signalr_name, replica_name, resource_group_name):
    return client.get(resource_group_name=resource_group_name, resource_name=signalr_name, replica_name=replica_name)


def signalr_replica_get(client: SignalRReplicasOperations, signalr_name, replica_name, resource_group_name):
    return client.get(resource_group_name=resource_group_name, resource_name=signalr_name, replica_name=replica_name)


def signalr_replica_start(client: SignalRReplicasOperations, signalr_name, replica_name, resource_group_name):
    parameter = Replica(location=None, resource_stopped=False)
    return client.begin_update(resource_group_name=resource_group_name, resource_name=signalr_name, replica_name=replica_name, parameters=parameter)


def signalr_replica_stop(client: SignalRReplicasOperations, signalr_name, replica_name, resource_group_name):
    parameter = Replica(location=None, resource_stopped=True)
    return client.begin_update(resource_group_name=resource_group_name, resource_name=signalr_name, replica_name=replica_name, parameters=parameter)


def signalr_replica_restart(client: SignalRReplicasOperations, signalr_name, replica_name, resource_group_name):
    return client.begin_restart(resource_group_name=resource_group_name, resource_name=signalr_name, replica_name=replica_name)


def signalr_replica_delete(client: SignalRReplicasOperations, signalr_name, replica_name, resource_group_name):
    return client.delete(resource_group_name=resource_group_name, resource_name=signalr_name, replica_name=replica_name)


def signalr_replica_create(client: SignalRReplicasOperations, signalr_name, replica_name, resource_group_name,
                           sku, unit_count=1, location=None, tags=None):
    sku = ResourceSku(name=sku, capacity=unit_count)
    parameter = Replica(tags=tags,
                        sku=sku,
                        location=location,
                        )

    return client.begin_create_or_update(resource_group_name=resource_group_name, resource_name=signalr_name,
                                         replica_name=replica_name, parameters=parameter)


def signalr_replica_set(client: SignalRReplicasOperations, signalr_name, replica_name, resource_group_name, parameters):
    return client.begin_update(resource_group_name=resource_group_name, resource_name=signalr_name,
                               replica_name=replica_name, parameters=parameters)


def signalr_replica_update(instance: Replica, unit_count=None, region_endpoint_enabled=None):
    unit_count = unit_count if unit_count else instance.sku.capacity
    instance.sku = ResourceSku(name=instance.sku.name, capacity=unit_count)

    if region_endpoint_enabled is not None:
        if region_endpoint_enabled:
            instance.region_endpoint_enabled = "Enabled"
        else:
            instance.region_endpoint_enabled = "Disabled"

    return instance



# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
#
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is
# regenerated.
# --------------------------------------------------------------------------
# pylint: disable=too-many-lines
# pylint: disable=too-many-statements

from azure.cli.core.commands.parameters import (
    tags_type,
    resource_group_name_type,
    get_location_type,
    get_enum_type
)
from azure.cli.core.commands.validators import get_default_location_from_resource_group


def load_arguments(self, _):

    with self.argument_context('sshkey list') as c:
        c.argument('resource_group_name', resource_group_name_type)

    with self.argument_context('sshkey show') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('ssh_public_key_name', options_list=['--name', '-n', '--ssh-public-key-name'], type=str, help='The '
                   'name of the SSH public key.', id_part='name')

    with self.argument_context('sshkey create') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('ssh_public_key_name', options_list=['--name', '-n', '--ssh-public-key-name'], type=str, help='The '
                   'name of the SSH public key.')
        c.argument('location', arg_type=get_location_type(self.cli_ctx), required=False,
                   validator=get_default_location_from_resource_group)
        c.argument('tags', tags_type)
        c.argument('public_key', type=str, help='SSH public key used to authenticate to a virtual machine through ssh. '
                   'If this property is not initially provided when the resource is created, the publicKey property '
                   'will be populated when generateKeyPair is called. If the public key is provided upon resource '
                   'creation, the provided public key needs to be at least 2048-bit and in ssh-rsa format.')
        c.argument('encryption_type', arg_type=get_enum_type(['RSA', 'Ed25519']), default='RSA', min_api='2023-09-01',
                   help='The encryption type of the SSH keys to be generated. ')

    with self.argument_context('sshkey update') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('ssh_public_key_name', options_list=['--name', '-n', '--ssh-public-key-name'], type=str, help='The '
                   'name of the SSH public key.', id_part='name')
        c.argument('tags', tags_type)
        c.argument('public_key', type=str, help='SSH public key used to authenticate to a virtual machine through ssh. '
                   'If this property is not initially provided when the resource is created, the publicKey property '
                   'will be populated when generateKeyPair is called. If the public key is provided upon resource '
                   'creation, the provided public key needs to be at least 2048-bit and in ssh-rsa format.')

    with self.argument_context('sshkey delete') as c:
        c.argument('resource_group_name', resource_group_name_type)
        c.argument('ssh_public_key_name', options_list=['--name', '-n', '--ssh-public-key-name'], type=str, help='The '
                   'name of the SSH public key.', id_part='name')



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=line-too-long
# pylint: disable=ungrouped-imports
# pylint: disable=unused-argument, unused-variable
# pylint: disable=broad-exception-caught
# pylint: disable=logging-format-interpolation
# pylint: disable=too-many-statements, too-many-locals, too-many-branches, too-many-public-methods, too-many-boolean-expressions, expression-not-assigned
from copy import deepcopy
from urllib.parse import urlparse

from typing import Dict, Any

from azure.cli.core.commands import AzCliCommand

from azure.cli.core.azclierror import (
    RequiredArgumentMissingError,
    ValidationError, ResourceNotFoundError, InvalidArgumentValueError)

from knack.log import get_logger

from ._constants import ACR_IMAGE_SUFFIX
from .base_resource import BaseResource
from ._client_factory import handle_raw_exception, handle_non_404_status_code_exception

from ._models import (
    RegistryCredentials as RegistryCredentialsModel)

from ._utils import (store_as_secret_and_return_secret_ref,
                     set_managed_identity,
                     safe_set,
                     safe_get,
                     _remove_registry_secret,
                     _get_acr_cred)

logger = get_logger(__name__)


class ContainerAppJobRegistryDecorator(BaseResource):
    def __init__(
            self, cmd: AzCliCommand, client: Any, raw_parameters: Dict, models: str
    ):
        super().__init__(cmd, client, raw_parameters, models)
        self.new_containerappjob = {}
        self.containerappjob_def = {}

    def get_argument_server(self):
        return self.get_param("server")

    def get_argument_username(self):
        return self.get_param("username")

    def set_argument_username(self, username):
        self.set_param("username", username)

    def get_argument_password(self):
        return self.get_param("password")

    def set_argument_password(self, password):
        self.set_param("password", password)

    def get_argument_identity(self):
        return self.get_param("identity")

    def get_argument_disable_warnings(self):
        return self.get_param("disable_warnings")

    def validate_arguments(self):
        # Check if containerapp job exists
        self.containerappjob_def = None
        try:
            self.containerappjob_def = self.client.show(cmd=self.cmd, resource_group_name=self.get_argument_resource_group_name(), name=self.get_argument_name())
        except Exception as e:
            handle_non_404_status_code_exception(e)

        if not self.containerappjob_def:
            raise ResourceNotFoundError("The containerapps job '{}' does not exist".format(self.get_argument_name()))

    def show(self):
        registries_def = safe_get(self.containerappjob_def, "properties", "configuration", "registries")
        if registries_def is None or len(registries_def) == 0:
            raise ValidationError("The containerapp job {} has no assigned registries.".format(self.get_argument_name()))

        for r in registries_def:
            if r['server'].lower() == self.get_argument_server().lower():
                return r
        raise InvalidArgumentValueError("The containerapp job {} does not have specified registry {} assigned.".format(self.get_argument_name(), self.get_argument_server()))

    def list(self):
        return safe_get(self.containerappjob_def, "properties", "configuration", "registries", default=[])

    def set_up_get_existing_secrets(self):
        if "secrets" not in self.containerappjob_def["properties"]["configuration"]:
            safe_set(self.containerappjob_def, "properties", "configuration", "secrets", value=[])
            safe_set(self.new_containerappjob, "properties", "configuration", "secrets", value=[])
        else:
            secrets = None
            try:
                secrets = self.client.list_secrets(cmd=self.cmd, resource_group_name=self.get_argument_resource_group_name(), name=self.get_argument_name())
            except Exception as e:  # pylint: disable=broad-except
                handle_raw_exception(e)
            safe_set(self.containerappjob_def, "properties", "configuration", "secrets", value=secrets["value"])
            safe_set(self.new_containerappjob, "properties", "configuration", "secrets", value=secrets["value"])

    def send_update(self):
        try:
            r = self.client.update(
                cmd=self.cmd, resource_group_name=self.get_argument_resource_group_name(), name=self.get_argument_name(), containerapp_job_envelope=self.new_containerappjob,
                no_wait=self.get_argument_no_wait())
            return r
        except Exception as e:
            handle_raw_exception(e)


class ContainerAppJobRegistryRemoveDecorator(ContainerAppJobRegistryDecorator):
    def construct_payload(self):
        registries_def = safe_get(self.containerappjob_def, "properties", "configuration", "registries")
        safe_set(self.new_containerappjob, "properties", "configuration", "registries", value=registries_def)
        logger.warning(self.new_containerappjob["properties"]["configuration"]["registries"])
        if registries_def is None or len(registries_def) == 0:
            raise ValidationError(
                "The containerapp job {} has no assigned registries.".format(self.get_argument_name()))

        self.set_up_get_existing_secrets()

        was_removed = False
        for i, value in enumerate(registries_def):
            r = value
            if r['server'].lower() == self.get_argument_server().lower():
                registries_def.pop(i)
                if r.get('username'):
                    _remove_registry_secret(containerapp_def=self.new_containerappjob, server=self.get_argument_server(),
                                            username=r.get('username'))
                was_removed = True
                break

        if not was_removed:
            raise ValidationError(
                "The containerapp job {} does not have specified registry {} assigned.".format(self.get_argument_name(),
                                                                                               self.get_argument_server()))

    def remove(self):
        r = self.send_update()
        logger.warning("Registry successfully removed.")
        return safe_get(r, "properties", "configuration", "registries")


class ContainerAppJobRegistrySetDecorator(ContainerAppJobRegistryDecorator):
    def validate_arguments(self):
        if (self.get_argument_username() or self.get_argument_password()) and self.get_argument_identity():
            raise ValidationError("Use either identity or --username/--password.")

        super().validate_arguments()

    def construct_payload(self):

        self.set_up_get_existing_secrets()

        registries_def = safe_get(self.containerappjob_def, "properties", "configuration", "registries", default=[])
        safe_set(self.new_containerappjob, "properties", "configuration", "registries", value=registries_def)

        if (not self.get_argument_username() or not self.get_argument_password()) and not self.get_argument_identity():
            # If registry is Azure Container Registry, we can try inferring credentials
            if ACR_IMAGE_SUFFIX not in self.get_argument_server():
                raise RequiredArgumentMissingError(
                    'Registry username and password are required if you are not using Azure Container Registry.')
            not self.get_argument_disable_warnings() and logger.warning(
                'No credential was provided to access Azure Container Registry. Trying to look up...')
            parsed = urlparse(self.get_argument_server())
            registry_name = (parsed.netloc if parsed.scheme else parsed.path).split('.')[0]

            try:
                username, password, _ = _get_acr_cred(self.cmd.cli_ctx, registry_name)
                self.set_argument_username(username)
                self.set_argument_password(password)
            except Exception as ex:
                raise RequiredArgumentMissingError(
                    'Failed to retrieve credentials for container registry. Please provide the registry username and password') from ex

        # Check if updating existing registry
        updating_existing_registry = False
        for r in registries_def:
            if r['server'].lower() == self.get_argument_server().lower():
                not self.get_argument_disable_warnings() and logger.warning("Updating existing registry.")
                updating_existing_registry = True
                if self.get_argument_username():
                    r["username"] = self.get_argument_username()
                    r["identity"] = None
                if self.get_argument_password():
                    r["passwordSecretRef"] = store_as_secret_and_return_secret_ref(
                        self.new_containerappjob["properties"]["configuration"]["secrets"],
                        r["username"],
                        r["server"],
                        self.get_argument_password(),
                        update_existing_secret=True)
                    r["identity"] = None
                if self.get_argument_identity():
                    r["identity"] = self.get_argument_identity()
                    r["username"] = None
                    r["passwordSecretRef"] = None

        # If not updating existing registry, add as new registry
        if not updating_existing_registry:
            registry = deepcopy(RegistryCredentialsModel)
            registry["server"] = self.get_argument_server()
            if not self.get_argument_identity():
                registry["username"] = self.get_argument_username()
                registry["passwordSecretRef"] = store_as_secret_and_return_secret_ref(
                    self.new_containerappjob["properties"]["configuration"]["secrets"],
                    self.get_argument_username(),
                    self.get_argument_server(),
                    self.get_argument_password(),
                    update_existing_secret=True)
            else:
                registry["identity"] = self.get_argument_identity()

            registries_def.append(registry)

        if self.get_argument_identity():
            identity_def = safe_get(self.containerappjob_def, "identity", default={})
            safe_set(self.new_containerappjob, "identity", value=identity_def)

            system_assigned_identity = self.get_argument_identity().lower() == "system"
            user_assigned = None if system_assigned_identity else [self.get_argument_identity()]
            set_managed_identity(self.cmd, self.get_argument_resource_group_name(), self.new_containerappjob, system_assigned_identity, user_assigned)

    def set(self):
        r = self.send_update()
        return safe_get(r, "properties", "configuration", "registries")



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# pylint: disable=redefined-builtin

import json
import requests
from azure.cli.core.util import CLIError
from azure.cli.command_modules.ams._completers import get_mru_type_completion_list
from azure.cli.core.commands.client_factory import get_subscription_id
from azure.cli.core.azclierror import BadRequestError

_rut_dict = {0: 'S1',
             1: 'S2',
             2: 'S3'}


def get_mru(client, cmd, resource_group_name, account_name):
    account_info = client.get(resource_group_name,
                              account_name) if resource_group_name else client.get_by_subscription(account_name)
    if account_info.encryption:
        raise BadRequestError('The media reserved unit operation failed as the Media Services account was created'
                              ' with the 2020-05-01 version of the API or later. Accounts created this way no'
                              ' longer need to set media reserved units as the system will automatically'
                              ' scale up and down based on load.')
    mru = MediaV2Client(cmd.cli_ctx, resource_group_name, account_name).get_mru()
    return _map_mru(mru)


def set_mru(client, cmd, resource_group_name, account_name, count=None, type=None):
    account_info = client.get(resource_group_name,
                              account_name) if resource_group_name else client.get_by_subscription(account_name)
    if account_info.encryption:
        raise BadRequestError('The media reserved unit operation failed as the Media Services account was created'
                              ' with the 2020-05-01 version of the API or later. Accounts created this way no'
                              ' longer need to set media reserved units as the system will automatically'
                              ' scale up and down based on load.')
    client = MediaV2Client(cmd.cli_ctx, resource_group_name, account_name)
    mru = client.get_mru()

    count = count if count is not None else int(mru['CurrentReservedUnits'])

    if type is None:
        type = int(mru['ReservedUnitType'])
    else:
        try:
            type = int(list(_rut_dict.keys())[list(_rut_dict.values()).index(type)])
        except:
            raise CLIError(
                'Invalid --type argument. Allowed values: {}.'.format(", ".join(get_mru_type_completion_list())))

    client.set_mru(mru['AccountId'], count, type)
    return _map_mru(client.get_mru())


def _map_mru(mru):
    mapped_obj = {}
    mapped_obj['count'] = mru['CurrentReservedUnits']
    mapped_obj['type'] = _rut_dict[mru['ReservedUnitType']]
    return mapped_obj


class MediaV2Client:
    """ Media V2 Client """
    def __init__(self, cli_ctx, resource_group_name, account_name):
        from azure.cli.core._profile import Profile
        self.profile = Profile(cli_ctx=cli_ctx)
        self._old_rp_api_version = '2015-10-01'
        self.v2_media_api_resource = cli_ctx.cloud.endpoints.media_resource_id
        self.api_endpoint = self._get_v2_api_endpoint(cli_ctx, resource_group_name, account_name)
        self.access_token = self._get_v2_access_token()

    def _get_v2_api_endpoint(self, cli_ctx, resource_group_name, account_name):
        from azure.mgmt.core.tools import resource_id
        from azure.cli.command_modules.ams._sdk_utils import (get_media_namespace, get_media_type)

        access_token = self.profile.get_raw_token()[0][2].get('accessToken')

        media_old_rp_url = resource_id(subscription=get_subscription_id(cli_ctx),
                                       resource_group=resource_group_name,
                                       namespace=get_media_namespace(), type=get_media_type(),
                                       name=account_name) + '?api-version={}'.format(self._old_rp_api_version)

        media_service_res = requests.get(cli_ctx.cloud.endpoints.resource_manager.rstrip('/') + media_old_rp_url,
                                         headers={'Authorization': 'Bearer {}'.format(access_token)})
        if not media_service_res.ok:
            err_info = 'Request to 2015-10-01 Media API failed.'
            res_text = json.loads(media_service_res.text)
            if res_text is not None and res_text.get('error') is not None:
                err_info = res_text.get('error').get('message')
            raise CLIError(err_info)

        media_service = media_service_res.json()
        api_endpoints = media_service.get('properties').get('apiEndpoints')
        api_endpoint = next((x for x in api_endpoints if x.get('majorVersion') == '2'), api_endpoints[0])
        if not api_endpoint:
            raise CLIError('v2 Media API endpoint was not found.')
        return api_endpoint

    def _get_v2_access_token(self):
        # pylint: disable=protected-access
        access_token = self.profile.get_raw_token(resource=self.v2_media_api_resource)[0][2].get('accessToken')
        return access_token

    def set_mru(self, account_id, count, type):
        headers = {}
        headers['Authorization'] = 'Bearer {}'.format(self.access_token)
        headers['Content-Type'] = 'application/json;odata=verbose'
        headers['Accept'] = 'application/json;odata=verbose'

        url_request_template = "{}EncodingReservedUnitTypes(guid'{}')?api-version=2.19"
        response = requests.put(url_request_template.format(self.api_endpoint.get('endpoint'), account_id),
                                headers=headers,
                                data='{{"ReservedUnitType":{}, "CurrentReservedUnits":{}}}'.format(type, count))

        if not response.ok:
            err_info = 'No error information available'
            if json.loads(response.text) is not None and json.loads(response.text).get('error') is not None:
                err_info = json.loads(response.text).get('error').get('message').get('value')
            raise CLIError('Request to EncodingReservedUnitTypes v2 API endpoint failed. ' + err_info)

    def get_mru(self):
        headers = {}
        headers['Authorization'] = 'Bearer {}'.format(self.access_token)
        headers['Content-Type'] = 'application/json;odata=minimalmetadata'
        headers['Accept'] = 'application/json;odata=minimalmetadata'
        headers['Accept-Charset'] = 'UTF-8'

        url_request_template = '{}EncodingReservedUnitTypes?api-version=2.19'
        response = requests.get(url_request_template.format(self.api_endpoint.get('endpoint')),
                                headers=headers)
        if not response.ok:
            raise CLIError('Request to EncodingReservedUnitTypes v2 API endpoint failed.')
        return response.json().get('value')[0]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
#
# Code generated by aaz-dev-tools
# --------------------------------------------------------------------------------------------

# pylint: skip-file
# flake8: noqa

from azure.cli.core.aaz import *


@register_command(
    "sig image-definition update",
)
class Update(AAZCommand):
    """Update a VM Image definition.

    :example: Change the shared image definition's recommended configuration
        az sig image-definition update --resource-group MyResourceGroup --gallery-name MyGallery --gallery-image-definition MyImage --resource-group MyResourceGroup --set recommended.vCpUs.min=myNewvCpUsMin recommended.vCpUs.max=myNewvCpUsMax recommended.memory.min=myNewMemoryMin recommended.memory.max=myNewMemoryMax description="newDescription"

    :example: Remove a shared image definition's configuration property
        az sig image-definition update --resource-group MyResourceGroup --gallery-name MyGallery --gallery-image-definition MyImage --resource-group MyResourceGroup --remove recommended.vCpUs.min
    """

    _aaz_info = {
        "version": "2021-10-01",
        "resources": [
            ["mgmt-plane", "/subscriptions/{}/resourcegroups/{}/providers/microsoft.compute/galleries/{}/images/{}", "2021-10-01"],
        ]
    }

    AZ_SUPPORT_NO_WAIT = True

    AZ_SUPPORT_GENERIC_UPDATE = True

    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, self._output)

    _args_schema = None

    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        if cls._args_schema is not None:
            return cls._args_schema
        cls._args_schema = super()._build_arguments_schema(*args, **kwargs)

        # define Arg Group ""

        _args_schema = cls._args_schema
        _args_schema.gallery_image_definition = AAZStrArg(
            options=["-i", "--gallery-image-definition"],
            help="The name of the gallery image definition to be retrieved.",
            required=True,
            id_part="child_name_1",
        )
        _args_schema.gallery_name = AAZStrArg(
            options=["-r", "--gallery-name"],
            help="The name of the Shared Image Gallery in which the Image Definition resides.",
            required=True,
            id_part="name",
        )
        _args_schema.resource_group = AAZResourceGroupNameArg(
            required=True,
        )

        # define Arg Group "GalleryImage"

        _args_schema = cls._args_schema
        _args_schema.location = AAZResourceLocationArg(
            arg_group="GalleryImage",
            help="Resource location",
            fmt=AAZResourceLocationArgFormat(
                resource_group_arg="resource_group",
            ),
        )
        _args_schema.tags = AAZDictArg(
            options=["--tags"],
            arg_group="GalleryImage",
            help="Resource tags",
            nullable=True,
        )

        tags = cls._args_schema.tags
        tags.Element = AAZStrArg(
            nullable=True,
        )

        # define Arg Group "Properties"

        _args_schema = cls._args_schema
        _args_schema.architecture = AAZStrArg(
            options=["--architecture"],
            arg_group="Properties",
            help="The architecture of the image. Applicable to OS disks only.",
            nullable=True,
            enum={"Arm64": "Arm64", "x64": "x64"},
        )
        _args_schema.description = AAZStrArg(
            options=["--description"],
            arg_group="Properties",
            help="The description of this gallery image definition resource. This property is updatable.",
            nullable=True,
        )
        _args_schema.disallowed = AAZObjectArg(
            options=["--disallowed"],
            arg_group="Properties",
            help="Describes the disallowed disk types.",
            nullable=True,
        )
        _args_schema.end_of_life_date = AAZDateTimeArg(
            options=["--end-of-life-date"],
            arg_group="Properties",
            help="The end of life date of the gallery image definition. This property can be used for decommissioning purposes. This property is updatable.",
            nullable=True,
        )
        _args_schema.eula = AAZStrArg(
            options=["--eula"],
            arg_group="Properties",
            help="The Eula agreement for the gallery image definition.",
            nullable=True,
        )
        _args_schema.features = AAZListArg(
            options=["--features"],
            arg_group="Properties",
            help="A list of gallery image features.",
            nullable=True,
        )
        _args_schema.hyper_v_generation = AAZStrArg(
            options=["--hyper-v-generation"],
            arg_group="Properties",
            help="The hypervisor generation of the Virtual Machine. Applicable to OS disks only.",
            nullable=True,
            enum={"V1": "V1", "V2": "V2"},
        )
        _args_schema.identifier = AAZObjectArg(
            options=["--identifier"],
            arg_group="Properties",
            help="This is the gallery image definition identifier.",
        )
        _args_schema.os_state = AAZStrArg(
            options=["--os-state"],
            arg_group="Properties",
            help="This property allows the user to specify whether the virtual machines created under this image are 'Generalized' or 'Specialized'.",
            enum={"Generalized": "Generalized", "Specialized": "Specialized"},
        )
        _args_schema.os_type = AAZStrArg(
            options=["--os-type"],
            arg_group="Properties",
            help="This property allows you to specify the type of the OS that is included in the disk when creating a VM from a managed image. <br><br> Possible values are: <br><br> **Windows** <br><br> **Linux**",
            enum={"Linux": "Linux", "Windows": "Windows"},
        )
        _args_schema.privacy_statement_uri = AAZStrArg(
            options=["--privacy-statement-uri"],
            arg_group="Properties",
            help="The privacy statement uri.",
            nullable=True,
        )
        _args_schema.purchase_plan = AAZObjectArg(
            options=["--purchase-plan"],
            arg_group="Properties",
            help="Describes the gallery image definition purchase plan. This is used by marketplace images.",
            nullable=True,
        )
        _args_schema.recommended = AAZObjectArg(
            options=["--recommended"],
            arg_group="Properties",
            help="The properties describe the recommended machine configuration for this Image Definition. These properties are updatable.",
            nullable=True,
        )
        _args_schema.release_note_uri = AAZStrArg(
            options=["--release-note-uri"],
            arg_group="Properties",
            help="The release note uri.",
            nullable=True,
        )

        disallowed = cls._args_schema.disallowed
        disallowed.disk_types = AAZListArg(
            options=["disk-types"],
            help="A list of disk types.",
            nullable=True,
        )

        disk_types = cls._args_schema.disallowed.disk_types
        disk_types.Element = AAZStrArg(
            nullable=True,
        )

        features = cls._args_schema.features
        features.Element = AAZObjectArg(
            nullable=True,
        )

        _element = cls._args_schema.features.Element
        _element.name = AAZStrArg(
            options=["name"],
            help="The name of the gallery image feature.",
            nullable=True,
        )
        _element.value = AAZStrArg(
            options=["value"],
            help="The value of the gallery image feature.",
            nullable=True,
        )

        identifier = cls._args_schema.identifier
        identifier.offer = AAZStrArg(
            options=["offer"],
            help="The name of the gallery image definition offer.",
        )
        identifier.publisher = AAZStrArg(
            options=["publisher"],
            help="The name of the gallery image definition publisher.",
        )
        identifier.sku = AAZStrArg(
            options=["sku"],
            help="The name of the gallery image definition SKU.",
        )

        purchase_plan = cls._args_schema.purchase_plan
        purchase_plan.name = AAZStrArg(
            options=["name"],
            help="The plan ID.",
            nullable=True,
        )
        purchase_plan.product = AAZStrArg(
            options=["product"],
            help="The product ID.",
            nullable=True,
        )
        purchase_plan.publisher = AAZStrArg(
            options=["publisher"],
            help="The publisher ID.",
            nullable=True,
        )

        recommended = cls._args_schema.recommended
        recommended.memory = AAZObjectArg(
            options=["memory"],
            nullable=True,
        )
        cls._build_args_resource_range_update(recommended.memory)
        recommended.v_cp_us = AAZObjectArg(
            options=["v-cp-us"],
            help="Describes the resource range.",
            nullable=True,
        )
        cls._build_args_resource_range_update(recommended.v_cp_us)
        return cls._args_schema

    _args_resource_range_update = None

    @classmethod
    def _build_args_resource_range_update(cls, _schema):
        if cls._args_resource_range_update is not None:
            _schema.max = cls._args_resource_range_update.max
            _schema.min = cls._args_resource_range_update.min
            return

        cls._args_resource_range_update = AAZObjectArg(
            nullable=True,
        )

        resource_range_update = cls._args_resource_range_update
        resource_range_update.max = AAZIntArg(
            options=["max"],
            help="The maximum number of the resource.",
            nullable=True,
        )
        resource_range_update.min = AAZIntArg(
            options=["min"],
            help="The minimum number of the resource.",
            nullable=True,
        )

        _schema.max = cls._args_resource_range_update.max
        _schema.min = cls._args_resource_range_update.min

    def _execute_operations(self):
        self.pre_operations()
        self.GalleryImagesGet(ctx=self.ctx)()
        self.pre_instance_update(self.ctx.vars.instance)
        self.InstanceUpdateByJson(ctx=self.ctx)()
        self.InstanceUpdateByGeneric(ctx=self.ctx)()
        self.post_instance_update(self.ctx.vars.instance)
        yield self.GalleryImagesCreateOrUpdate(ctx=self.ctx)()
        self.post_operations()

    @register_callback
    def pre_operations(self):
        pass

    @register_callback
    def post_operations(self):
        pass

    @register_callback
    def pre_instance_update(self, instance):
        pass

    @register_callback
    def post_instance_update(self, instance):
        pass

    def _output(self, *args, **kwargs):
        result = self.deserialize_output(self.ctx.vars.instance, client_flatten=True)
        return result

    class GalleryImagesGet(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [200]:
                return self.on_200(session)

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/galleries/{galleryName}/images/{galleryImageName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "GET"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "galleryImageName", self.ctx.args.gallery_image_definition,
                    required=True,
                ),
                **self.serialize_url_param(
                    "galleryName", self.ctx.args.gallery_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2021-10-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        def on_200(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200
            )

        _schema_on_200 = None

        @classmethod
        def _build_schema_on_200(cls):
            if cls._schema_on_200 is not None:
                return cls._schema_on_200

            cls._schema_on_200 = AAZObjectType()
            _UpdateHelper._build_schema_gallery_image_read(cls._schema_on_200)

            return cls._schema_on_200

    class GalleryImagesCreateOrUpdate(AAZHttpOperation):
        CLIENT_TYPE = "MgmtClient"

        def __call__(self, *args, **kwargs):
            request = self.make_request()
            session = self.client.send_request(request=request, stream=False, **kwargs)
            if session.http_response.status_code in [202]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )
            if session.http_response.status_code in [200, 201]:
                return self.client.build_lro_polling(
                    self.ctx.args.no_wait,
                    session,
                    self.on_200_201,
                    self.on_error,
                    lro_options={"final-state-via": "azure-async-operation"},
                    path_format_arguments=self.url_parameters,
                )

            return self.on_error(session.http_response)

        @property
        def url(self):
            return self.client.format_url(
                "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Compute/galleries/{galleryName}/images/{galleryImageName}",
                **self.url_parameters
            )

        @property
        def method(self):
            return "PUT"

        @property
        def error_format(self):
            return "ODataV4Format"

        @property
        def url_parameters(self):
            parameters = {
                **self.serialize_url_param(
                    "galleryImageName", self.ctx.args.gallery_image_definition,
                    required=True,
                ),
                **self.serialize_url_param(
                    "galleryName", self.ctx.args.gallery_name,
                    required=True,
                ),
                **self.serialize_url_param(
                    "resourceGroupName", self.ctx.args.resource_group,
                    required=True,
                ),
                **self.serialize_url_param(
                    "subscriptionId", self.ctx.subscription_id,
                    required=True,
                ),
            }
            return parameters

        @property
        def query_parameters(self):
            parameters = {
                **self.serialize_query_param(
                    "api-version", "2021-10-01",
                    required=True,
                ),
            }
            return parameters

        @property
        def header_parameters(self):
            parameters = {
                **self.serialize_header_param(
                    "Content-Type", "application/json",
                ),
                **self.serialize_header_param(
                    "Accept", "application/json",
                ),
            }
            return parameters

        @property
        def content(self):
            _content_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=self.ctx.vars.instance,
            )

            return self.serialize_content(_content_value)

        def on_200_201(self, session):
            data = self.deserialize_http_content(session)
            self.ctx.set_var(
                "instance",
                data,
                schema_builder=self._build_schema_on_200_201
            )

        _schema_on_200_201 = None

        @classmethod
        def _build_schema_on_200_201(cls):
            if cls._schema_on_200_201 is not None:
                return cls._schema_on_200_201

            cls._schema_on_200_201 = AAZObjectType()
            _UpdateHelper._build_schema_gallery_image_read(cls._schema_on_200_201)

            return cls._schema_on_200_201

    class InstanceUpdateByJson(AAZJsonInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance(self.ctx.vars.instance)

        def _update_instance(self, instance):
            _instance_value, _builder = self.new_content_builder(
                self.ctx.args,
                value=instance,
                typ=AAZObjectType
            )
            _builder.set_prop("location", AAZStrType, ".location", typ_kwargs={"flags": {"required": True}})
            _builder.set_prop("properties", AAZObjectType, typ_kwargs={"flags": {"client_flatten": True}})
            _builder.set_prop("tags", AAZDictType, ".tags")

            properties = _builder.get(".properties")
            if properties is not None:
                properties.set_prop("architecture", AAZStrType, ".architecture")
                properties.set_prop("description", AAZStrType, ".description")
                properties.set_prop("disallowed", AAZObjectType, ".disallowed")
                properties.set_prop("endOfLifeDate", AAZStrType, ".end_of_life_date")
                properties.set_prop("eula", AAZStrType, ".eula")
                properties.set_prop("features", AAZListType, ".features")
                properties.set_prop("hyperVGeneration", AAZStrType, ".hyper_v_generation")
                properties.set_prop("identifier", AAZObjectType, ".identifier", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("osState", AAZStrType, ".os_state", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("osType", AAZStrType, ".os_type", typ_kwargs={"flags": {"required": True}})
                properties.set_prop("privacyStatementUri", AAZStrType, ".privacy_statement_uri")
                properties.set_prop("purchasePlan", AAZObjectType, ".purchase_plan")
                properties.set_prop("recommended", AAZObjectType, ".recommended")
                properties.set_prop("releaseNoteUri", AAZStrType, ".release_note_uri")

            disallowed = _builder.get(".properties.disallowed")
            if disallowed is not None:
                disallowed.set_prop("diskTypes", AAZListType, ".disk_types")

            disk_types = _builder.get(".properties.disallowed.diskTypes")
            if disk_types is not None:
                disk_types.set_elements(AAZStrType, ".")

            features = _builder.get(".properties.features")
            if features is not None:
                features.set_elements(AAZObjectType, ".")

            _elements = _builder.get(".properties.features[]")
            if _elements is not None:
                _elements.set_prop("name", AAZStrType, ".name")
                _elements.set_prop("value", AAZStrType, ".value")

            identifier = _builder.get(".properties.identifier")
            if identifier is not None:
                identifier.set_prop("offer", AAZStrType, ".offer", typ_kwargs={"flags": {"required": True}})
                identifier.set_prop("publisher", AAZStrType, ".publisher", typ_kwargs={"flags": {"required": True}})
                identifier.set_prop("sku", AAZStrType, ".sku", typ_kwargs={"flags": {"required": True}})

            purchase_plan = _builder.get(".properties.purchasePlan")
            if purchase_plan is not None:
                purchase_plan.set_prop("name", AAZStrType, ".name")
                purchase_plan.set_prop("product", AAZStrType, ".product")
                purchase_plan.set_prop("publisher", AAZStrType, ".publisher")

            recommended = _builder.get(".properties.recommended")
            if recommended is not None:
                _UpdateHelper._build_schema_resource_range_update(recommended.set_prop("memory", AAZObjectType, ".memory"))
                _UpdateHelper._build_schema_resource_range_update(recommended.set_prop("vCPUs", AAZObjectType, ".v_cp_us"))

            tags = _builder.get(".tags")
            if tags is not None:
                tags.set_elements(AAZStrType, ".")

            return _instance_value

    class InstanceUpdateByGeneric(AAZGenericInstanceUpdateOperation):

        def __call__(self, *args, **kwargs):
            self._update_instance_by_generic(
                self.ctx.vars.instance,
                self.ctx.generic_update_args
            )


class _UpdateHelper:
    """Helper class for Update"""

    @classmethod
    def _build_schema_resource_range_update(cls, _builder):
        if _builder is None:
            return
        _builder.set_prop("max", AAZIntType, ".max")
        _builder.set_prop("min", AAZIntType, ".min")

    _schema_gallery_image_read = None

    @classmethod
    def _build_schema_gallery_image_read(cls, _schema):
        if cls._schema_gallery_image_read is not None:
            _schema.id = cls._schema_gallery_image_read.id
            _schema.location = cls._schema_gallery_image_read.location
            _schema.name = cls._schema_gallery_image_read.name
            _schema.properties = cls._schema_gallery_image_read.properties
            _schema.tags = cls._schema_gallery_image_read.tags
            _schema.type = cls._schema_gallery_image_read.type
            return

        cls._schema_gallery_image_read = _schema_gallery_image_read = AAZObjectType()

        gallery_image_read = _schema_gallery_image_read
        gallery_image_read.id = AAZStrType(
            flags={"read_only": True},
        )
        gallery_image_read.location = AAZStrType(
            flags={"required": True},
        )
        gallery_image_read.name = AAZStrType(
            flags={"read_only": True},
        )
        gallery_image_read.properties = AAZObjectType(
            flags={"client_flatten": True},
        )
        gallery_image_read.tags = AAZDictType()
        gallery_image_read.type = AAZStrType(
            flags={"read_only": True},
        )

        properties = _schema_gallery_image_read.properties
        properties.architecture = AAZStrType()
        properties.description = AAZStrType()
        properties.disallowed = AAZObjectType()
        properties.end_of_life_date = AAZStrType(
            serialized_name="endOfLifeDate",
        )
        properties.eula = AAZStrType()
        properties.features = AAZListType()
        properties.hyper_v_generation = AAZStrType(
            serialized_name="hyperVGeneration",
        )
        properties.identifier = AAZObjectType(
            flags={"required": True},
        )
        properties.os_state = AAZStrType(
            serialized_name="osState",
            flags={"required": True},
        )
        properties.os_type = AAZStrType(
            serialized_name="osType",
            flags={"required": True},
        )
        properties.privacy_statement_uri = AAZStrType(
            serialized_name="privacyStatementUri",
        )
        properties.provisioning_state = AAZStrType(
            serialized_name="provisioningState",
            flags={"read_only": True},
        )
        properties.purchase_plan = AAZObjectType(
            serialized_name="purchasePlan",
        )
        properties.recommended = AAZObjectType()
        properties.release_note_uri = AAZStrType(
            serialized_name="releaseNoteUri",
        )

        disallowed = _schema_gallery_image_read.properties.disallowed
        disallowed.disk_types = AAZListType(
            serialized_name="diskTypes",
        )

        disk_types = _schema_gallery_image_read.properties.disallowed.disk_types
        disk_types.Element = AAZStrType()

        features = _schema_gallery_image_read.properties.features
        features.Element = AAZObjectType()

        _element = _schema_gallery_image_read.properties.features.Element
        _element.name = AAZStrType()
        _element.value = AAZStrType()

        identifier = _schema_gallery_image_read.properties.identifier
        identifier.offer = AAZStrType(
            flags={"required": True},
        )
        identifier.publisher = AAZStrType(
            flags={"required": True},
        )
        identifier.sku = AAZStrType(
            flags={"required": True},
        )

        purchase_plan = _schema_gallery_image_read.properties.purchase_plan
        purchase_plan.name = AAZStrType()
        purchase_plan.product = AAZStrType()
        purchase_plan.publisher = AAZStrType()

        recommended = _schema_gallery_image_read.properties.recommended
        recommended.memory = AAZObjectType()
        cls._build_schema_resource_range_read(recommended.memory)
        recommended.v_cp_us = AAZObjectType(
            serialized_name="vCPUs",
        )
        cls._build_schema_resource_range_read(recommended.v_cp_us)

        tags = _schema_gallery_image_read.tags
        tags.Element = AAZStrType()

        _schema.id = cls._schema_gallery_image_read.id
        _schema.location = cls._schema_gallery_image_read.location
        _schema.name = cls._schema_gallery_image_read.name
        _schema.properties = cls._schema_gallery_image_read.properties
        _schema.tags = cls._schema_gallery_image_read.tags
        _schema.type = cls._schema_gallery_image_read.type

    _schema_resource_range_read = None

    @classmethod
    def _build_schema_resource_range_read(cls, _schema):
        if cls._schema_resource_range_read is not None:
            _schema.max = cls._schema_resource_range_read.max
            _schema.min = cls._schema_resource_range_read.min
            return

        cls._schema_resource_range_read = _schema_resource_range_read = AAZObjectType()

        resource_range_read = _schema_resource_range_read
        resource_range_read.max = AAZIntType()
        resource_range_read.min = AAZIntType()

        _schema.max = cls._schema_resource_range_read.max
        _schema.min = cls._schema_resource_range_read.min


__all__ = ["Update"]



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from datetime import datetime
from enum import Enum

from ._constants import SnapshotFilterFields

# pylint: disable=too-few-public-methods
# pylint: disable=too-many-instance-attributes
# pylint: disable=line-too-long


class SnapshotQueryFields(Enum):
    NAME = 0x0001
    STATUS = 0x0002
    FILTERS = 0x0004
    COMPOSITION_TYPE = 0x0008
    CREATED = 0x0010
    EXPIRES = 0x0020
    SIZE = 0x0040
    ITEMS_COUNT = 0x0080
    TAGS = 0x0100
    ETAG = 0x0200
    RETENTION_PERIOD = 0x0400
    ALL = NAME | STATUS | FILTERS | COMPOSITION_TYPE | CREATED | EXPIRES | SIZE | ITEMS_COUNT | TAGS | ETAG | RETENTION_PERIOD


class Snapshot:
    '''
    Class used to represent the Snapshot entity returned from requests

    :ivar str name:
        Name of the snapshot.
    :ivar str status:
        Provisioning status of the snapshot.
    :ivar dict[str, str] filters:
        Dictionary of filters used in building snapshot.
    :ivar str etag:
        The ETag contains a value that you can use to perform operations.
    :ivar str composition_type:
        Choose how snapshot filters are applied in building snapshot.
    :ivar str created:
        Time of creation of snapshot.
    :ivar str expires:
        The time at which an archived snapshot will be deleted.
    :ivar int size:
        The size in bytes of the built snapshot.
    :ivar int items_count:
        The number of key-values in the built snapshot.
    :ivar dict[str, str] tags:
        Dictionary of tags of the snapshot.
    :ivar int retention_period:
        Number of seconds for which an archived snapshot will be kept before being deleted.
    '''

    def __init__(self,
                 name,
                 status,
                 filters,
                 etag=None,
                 composition_type=None,
                 created=None,
                 expires=None,
                 size=None,
                 items_count=None,
                 tags=None,
                 retention_period=None,
                 ):

        self.name = name
        self.status = status
        self.filters = filters
        self.etag = etag
        self.composition_type = composition_type
        self.created = created.isoformat() if isinstance(created, datetime) else str(created)
        self.expires = expires.isoformat() if isinstance(expires, datetime) else (str(expires) if expires else None)
        self.size = size
        self.items_count = items_count
        self.tags = tags
        self.retention_period = retention_period

    def __str__(self):
        return "\nEtag: " + self.etag + \
            "\nName: " + self.name + \
            "\nStatus: " + self.status + \
            "\nFilters: " + (str(self.filters) if self.filters else '[]') + \
            "\nComposition Type: " + self.composition_type + \
            "\nCreated: " + self.created + \
            "\nExpires: " + self.expires + \
            "\nSize: " + str(self.size) + \
            "\nItem count: " + str(self.items_count) + \
            "\nTags: " + (str(self.tags) if self.tags else '{}') + \
            "\nRetention Period: " + str(self.retention_period)

    @classmethod
    def from_configuration_snapshot(cls, config_snapshot):
        return cls(
            name=config_snapshot.name,
            status=config_snapshot.status,
            filters=[convert_configuration_setting_filter(setting_filter) for setting_filter in config_snapshot.filters],
            etag=config_snapshot.etag,
            composition_type=config_snapshot.composition_type,
            created=config_snapshot.created,
            expires=config_snapshot.expires,
            size=config_snapshot.size,
            items_count=config_snapshot.items_count,
            tags=config_snapshot.tags,
            retention_period=config_snapshot.retention_period
        )


def convert_configuration_setting_filter(configuration_setting_filter):
    result = {}

    if configuration_setting_filter.key is not None:
        result[SnapshotFilterFields.KEY] = configuration_setting_filter.key

    if configuration_setting_filter.label is not None:
        result[SnapshotFilterFields.LABEL] = configuration_setting_filter.label

    if configuration_setting_filter.tags is not None:
        result[SnapshotFilterFields.TAGS] = configuration_setting_filter.tags

    return result



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------
# pylint: disable=too-many-locals, too-many-statements too-many-boolean-expressions too-many-branches protected-access

from azure.mgmt.cdn.models import (MinimumTlsVersion, ProtocolType, SkuName, UpdateRule, DeleteRule, CertificateType,
                                   ResourceType)
from azure.cli.core.aaz._base import has_value
from azure.cli.command_modules.cdn.aaz.latest.cdn.custom_domain import EnableHttps as _EnableHttps, \
    Delete as _CDNCustomDomainDelete
from azure.cli.command_modules.cdn.aaz.latest.afd.profile import Show as _AFDProfileShow, \
    Create as _AFDProfileCreate, Update as _AFDProfileUpdate, Delete as _AFDProfileDelete, \
    List as _AFDProfileList
from azure.cli.core.aaz import AAZStrArg, AAZBoolArg, AAZIntArg, AAZListArg, AAZTimeArg
from azure.cli.command_modules.cdn.aaz.latest.cdn.origin import Create as _CDNOriginCreate, \
    Update as _CDNOriginUpdate
from azure.cli.command_modules.cdn.aaz.latest.cdn.origin_group import Create as _CDNOriginGroupCreate, \
    Update as _CDNOriginGroupUpdate, Show as _CDNOriginGroupShow
from azure.cli.command_modules.cdn.aaz.latest.cdn.endpoint import Create as _CDNEndpointCreate, \
    Update as _CDNEndpointUpdate, Show as _CDNEndpointShow
from azure.cli.command_modules.cdn.aaz.latest.cdn.profile_migration import Migrate as _Migrate
from azure.cli.command_modules.cdn.aaz.latest.cdn._name_exists import NameExists
from .custom_rule_util import (create_condition, create_action, create_delivery_policy_from_existing)
import argparse

from knack.util import CLIError
from knack.log import get_logger

logger = get_logger(__name__)


def default_content_types():
    return ["text/plain",
            "text/html",
            "text/css",
            "text/javascript",
            "application/x-javascript",
            "application/javascript",
            "application/json",
            "application/xml"]


def _parse_ranges(ranges: str):
    if ranges is None:
        return []

    from azure.mgmt.cdn.models import HttpErrorRangeParameters

    def parse_range(error_range: str):
        split = error_range.split('-')
        if not split or len(split) > 2:
            raise CLIError(f'range "{error_range}" is invalid')

        try:
            begin = split[0]
            end = split[1] if len(split) == 2 else begin
        except ValueError:
            raise CLIError(f'range "{error_range}" is invalid')

        return HttpErrorRangeParameters(being=begin, end=end)

    return [parse_range(error_range) for error_range in ranges.split(',')]


class NameExistsWithType(NameExists):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        args.type = ResourceType.MICROSOFT_CDN_PROFILES_ENDPOINTS.value


class CDNProfileList(_AFDProfileList):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        return args_schema


class CDNProfileShow(_AFDProfileShow):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        return args_schema


class CDNProfileCreate(_AFDProfileCreate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.location._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        args.location = 'global'


class CDNProfileUpdate(_AFDProfileUpdate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.sku._registered = False
        return args_schema


class CDNProfileDelete(_AFDProfileDelete):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        return args_schema


class CDNEnableHttps(_EnableHttps):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.min_tls_version = AAZStrArg(
            options=['--min-tls-version'],
            help='The minimum TLS version required for the custom domain.',
        )
        args_schema.user_cert_group_name = AAZStrArg(
            options=['--user-cert-group-name'],
            help='The resource group of the KeyVault certificate.',
        )
        args_schema.user_cert_protocol_type = AAZStrArg(
            options=['--user-cert-protocol-type'],
            help='The protocol type of the certificate.',
        )
        args_schema.user_cert_secret_name = AAZStrArg(
            options=['--user-cert-secret-name'],
            help='The secret name of the KeyVault certificate.',
        )
        args_schema.user_cert_secret_version = AAZStrArg(
            options=['--user-cert-secret-version'],
            help='The secret version of the KeyVault certificate, '
            'If not specified, the "Latest" version will always been used and '
            'the deployed certificate will be automatically rotated to the latest version '
            'when a newer version of the certificate is available.',
        )
        args_schema.user_cert_subscription_id = AAZStrArg(
            options=['--user-cert-subscription-id'],
            help='The subscription id of the KeyVault certificate.',
        )
        args_schema.user_cert_vault_name = AAZStrArg(
            options=['--user-cert-vault-name'],
            help='The vault name of the KeyVault certificate.',
        )
        args_schema.minimum_tls_version._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        profile = CDNProfileShow(cli_ctx=self.cli_ctx)(command_args={
            'resource_group': args.resource_group,
            'profile_name': args.profile_name
        })

        if has_value(args.min_tls_version) and \
                args.min_tls_version.to_serialized_data().casefold() == 'none'.casefold():
            args.minimum_tls_version = MinimumTlsVersion.none
        elif args.min_tls_version == '1.0':
            args.minimum_tls_version = MinimumTlsVersion.tls10
        elif args.min_tls_version == '1.2':
            args.minimum_tls_version = MinimumTlsVersion.tls12

        # Are we using BYOC?
        if has_value(args.user_cert_group_name) or has_value(args.user_cert_protocol_type) or \
                has_value(args.user_cert_secret_name) or has_value(args.user_cert_secret_version) or \
                has_value(args.user_cert_subscription_id) or has_value(args.user_cert_vault_name):
            # If any BYOC flags are set, make sure they all are (except secret version).
            if not has_value(args.user_cert_group_name) or not has_value(args.user_cert_protocol_type) or \
                    not has_value(args.user_cert_secret_name) or \
                    not has_value(args.user_cert_vault_name):
                # BYOC is enabled, so make sure the secret version is set to None.
                raise CLIError("--user-cert-group-name, --user-cert-vault-name, --user-cert-secret-name, "
                               "and --user-cert-protocol-type are all required for user managed certificates.")
            if not has_value(args.user_cert_subscription_id):
                args.user_cert_subscription_id = self.ctx.subscription_id
            # All BYOC params are set, let's create the https parameters
            if not has_value(args.user_cert_protocol_type) or \
                    args.user_cert_protocol_type.to_serialized_data().lower() == 'sni':
                args.user_cert_protocol_type = ProtocolType.server_name_indication
            elif args.user_cert_protocol_type.to_serialized_data().lower() == 'ip':
                args.user_cert_protocol_type = ProtocolType.ip_based
            else:
                raise CLIError("--user-cert-protocol-type must be either 'sni' or 'ip'.")

            azure_key_vault = {
                'certificate_source_parameters': {
                    'delete_rule': DeleteRule.NO_ACTION,
                    'resource_group_name': args.user_cert_group_name,
                    'secret_name': args.user_cert_secret_name,
                    'subscription_id': args.user_cert_subscription_id,
                    'type_name': 'KeyVaultCertificateSourceParameters',
                    'update_rule': UpdateRule.NO_ACTION,
                    'vault_name': args.user_cert_vault_name,
                    'secret_version': args.user_cert_secret_version,
                }
            }

            args.azure_key_vault = azure_key_vault
            args.protocol_type = args.user_cert_protocol_type
        else:
            # We're using a CDN-managed certificate, let's create the right https
            # parameters for the profile SKU

            # Microsoft parameters
            if profile['sku']['name'] == SkuName.standard_microsoft:
                cdn = {
                    'certificate_source_parameters': {
                        'certificate_type': CertificateType.dedicated,
                        'type_name': 'CdnCertificateSourceParameters',
                    }
                }
                args.cdn = cdn
                args.protocol_type = ProtocolType.server_name_indication
            # Akamai parameters
            elif profile['sku']['name'] == SkuName.standard_akamai:
                cdn = {
                    'certificate_source_parameters': {
                        'certificate_type': CertificateType.shared,
                        'type_name': 'CdnCertificateSourceParameters',
                    }
                }
                args.cdn = cdn
                args.protocol_type = ProtocolType.server_name_indication
            # Verizon parameters
            else:
                cdn = {
                    'certificate_source_parameters': {
                        'certificate_type': CertificateType.shared,
                        'type_name': 'CdnCertificateSourceParameters',
                    }
                }
                args.cdn = cdn
                args.protocol_type = ProtocolType.ip_based


class CDNCustomDomainDelete(_CDNCustomDomainDelete):
    def _handler(self, command_args):
        super()._handler(command_args)
        return self.build_lro_poller(self._execute_operations, None)


class CDNOriginCreate(_CDNOriginCreate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.disabled = AAZBoolArg(
            options=['--disabled'],
            help='Don\'t use the origin for load balancing.',
            blank=True
        )
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        if not has_value(args.disabled):
            args.disabled = False
        args.enabled = not args.disabled
        if not has_value(args.http_port):
            args.http_port = 80
        if not has_value(args.https_port):
            args.https_port = 443
        if not has_value(args.priority):
            args.priority = 1
        elif int(args.priority.to_serialized_data()) < 1 or int(args.priority.to_serialized_data()) > 1000:
            raise CLIError('Priority must be between 1 and 1000')
        if not has_value(args.weight):
            args.weight = 1000


class CDNOriginUpdate(_CDNOriginUpdate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.disabled = AAZBoolArg(
            options=['--disabled'],
            help='Don\'t use the origin for load balancing.',
            blank=True
        )
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        if not has_value(args.disabled):
            args.disabled = False
        args.enabled = not args.disabled


class CDNOriginGroupCreate(_CDNOriginGroupCreate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.probe_interval = AAZIntArg(
            options=['--probe-interval'],
            help='The frequency to perform health probes in seconds.',
            blank=240
        )
        args_schema.probe_method = AAZStrArg(
            options=['--probe-method'],
            help='The request method to use for health probes.',
            blank='HEAD'
        )
        args_schema.probe_path = AAZStrArg(
            options=['--probe-path'],
            help='The path relative to the origin that is used to determine the health of the origin.',
        )
        args_schema.probe_protocol = AAZStrArg(
            options=['--probe-protocol'],
            help='The protocol to use for health probes.',
            blank='Http'
        )
        args_schema.origins = AAZStrArg(
            options=['--origins'],
            help='The origins load balanced by this origin group, '
            'as a comma-separated list of origin names or origin resource IDs.',
        )
        args_schema.response_error_detection_status_code_ranges = AAZStrArg(
            options=['--response-error-detection-status-code-ranges'],
            help='Type of response errors for real user requests for which origin will be deemed unhealthy',
        )
        args_schema.response_error_detection_failover_threshold = AAZIntArg(
            options=['--response-error-detection-failover-threshold'],
            help='The percentage of failed requests in the sample where failover should trigger.',
        )
        args_schema.response_error_detection_error_types = AAZStrArg(
            options=['--response-error-detection-error-types'],
            help='The list of Http status code ranges '
            'that are considered as server errors for origin and it is marked as unhealthy.',
        )
        return args_schema

    def pre_operations(self):
        args = self.ctx.args

        health_probe_settings = {
            'probe_interval_in_seconds': args.probe_interval,
            'probe_request_type': args.probe_method,
            'probe_path': args.probe_path,
            'probe_protocol': args.probe_protocol,
        }
        args.health_probe_settings = health_probe_settings

        formatted_origins = []
        if has_value(args.origins):
            for origin in args.origins.to_serialized_data().split(','):
                # If the origin is not an ID, assume it's a name and format it as an ID.
                if '/' not in origin:
                    origin = f'/subscriptions/{self.ctx.subscription_id}/resourceGroups/{args.resource_group}' \
                             f'/providers/Microsoft.Cdn/profiles/{args.profile_name}/endpoints/{args.endpoint_name}' \
                             f'/origins/{origin}'
                formatted_origins.append({'id': origin})
        args.formatted_origins = formatted_origins

        response_based_origin_error_detection_settings = None
        if has_value(args.response_error_detection_error_types) or \
           has_value(args.response_error_detection_failover_threshold) or \
           has_value(args.response_error_detection_status_code_ranges):
            response_based_origin_error_detection_settings = {
                'http_error_ranges': _parse_ranges(args.response_error_detection_status_code_ranges),
                'response_based_detected_error_types': args.response_error_detection_error_types,
                'response_based_failover_threshold_percentage': args.response_error_detection_failover_threshold
            }
        args.response_based_origin_error_detection_settings = response_based_origin_error_detection_settings


class CDNOriginGroupUpdate(_CDNOriginGroupUpdate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.probe_interval = AAZIntArg(
            options=['--probe-interval'],
            help='The frequency to perform health probes in seconds.',
        )
        args_schema.probe_method = AAZStrArg(
            options=['--probe-method'],
            help='The request method to use for health probes.',
        )
        args_schema.probe_path = AAZStrArg(
            options=['--probe-path'],
            help='The path relative to the origin that is used to determine the health of the origin.',
        )
        args_schema.probe_protocol = AAZStrArg(
            options=['--probe-protocol'],
            help='The protocol to use for health probes.',
        )
        args_schema.origins = AAZStrArg(
            options=['--origins'],
            help='The origins load balanced by this origin group, '
            'as a comma-separated list of origin names or origin resource IDs.',
        )
        args_schema.response_error_detection_status_code_ranges = AAZStrArg(
            options=['--response-error-detection-status-code-ranges'],
            help='Type of response errors for real user requests for which origin will be deemed unhealthy',
        )
        args_schema.response_error_detection_failover_threshold = AAZIntArg(
            options=['--response-error-detection-failover-threshold'],
            help='The percentage of failed requests in the sample where failover should trigger.',
        )
        args_schema.response_error_detection_error_types = AAZStrArg(
            options=['--response-error-detection-error-types'],
            help='The list of Http status code ranges '
            'that are considered as server errors for origin and it is marked as unhealthy.',
        )
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        existing = _CDNOriginGroupShow(cli_ctx=self.cli_ctx)(command_args={
            'resource_group': args.resource_group,
            'profile_name': args.profile_name,
            'endpoint_name': args.endpoint_name,
            'origin_group_name': args.origin_group_name
        })
        # Allow removing properties explicitly by specifying as empty string, or
        # update without modifying by not specifying (value is None).
        if not has_value(args.probe_path):
            args.probe_path = existing['healthProbeSettings']['probePath']
        elif args.probe_path.to_serialized_data() == '':
            args.probe_path = None
        if not has_value(args.probe_method):
            args.probe_method = existing['healthProbeSettings']['probeRequestType']
        elif args.probe_method.to_serialized_data() == '':
            args.probe_method = None
        if not has_value(args.probe_protocol):
            args.probe_protocol = existing['healthProbeSettings']['probeProtocol']
        elif args.probe_protocol.to_serialized_data() == '':
            args.probe_protocol = None
        if not has_value(args.probe_interval):
            args.probe_interval = existing['healthProbeSettings']['probeIntervalInSeconds']
        elif args.probe_interval.to_serialized_data() == '':
            args.probe_interval = None
        health_probe_settings = {
            'probe_interval_in_seconds': args.probe_interval,
            'probe_request_type': args.probe_method,
            'probe_path': args.probe_path,
            'probe_protocol': args.probe_protocol,
        }
        args.health_probe_settings = health_probe_settings

        formatted_origins = []
        if has_value(args.origins):
            args.origins = args.origins.to_serialized_data()
        else:
            args.origins = existing['origins']
        for origin in str(args.origins).split(','):
            # If the origin is not an ID, assume it's a name and format it as an ID.
            if '/' not in origin:
                origin = f'/subscriptions/{self.ctx.subscription_id}/resourceGroups/{args.resource_group}' \
                         f'/providers/Microsoft.Cdn/profiles/{args.profile_name}/endpoints/{args.endpoint_name}' \
                         f'/origins/{origin}'
            formatted_origins.append({'id': origin})
        args.formatted_origins = formatted_origins

        response_based_origin_error_detection_settings = None
        if has_value(args.response_error_detection_error_types) or \
           has_value(args.response_error_detection_failover_threshold) or \
           has_value(args.response_error_detection_status_code_ranges):
            response_based_origin_error_detection_settings = {
                'http_error_ranges': _parse_ranges(args.response_error_detection_status_code_ranges),
                'response_based_detected_error_types': args.response_error_detection_error_types,
                'response_based_failover_threshold_percentage': args.response_error_detection_failover_threshold
            }
        args.response_based_origin_error_detection_settings = response_based_origin_error_detection_settings


class CDNEndpointCreate(_CDNEndpointCreate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.origin = AAZListArg(
            options=['--origin'],
            help='Endpoint origin specified by the following space-delimited 7 tuple: '
            'www.example.com http_port https_port private_link_resource_id '
            'private_link_location private_link_approval_message origin_name. '
            'The HTTP and HTTPS ports and the private link resource ID and location are optional. '
            'The HTTP and HTTPS ports default to 80 and 443, respectively. '
            'Private link fields are only valid for the sku Standard_Microsoft, '
            'and private_link_location is required if private_link_resource_id is set. '
            'the origin name is optional and defaults to origin.',
            required=True,
        )
        args_schema.origin.Element = AAZStrArg()
        args_schema.no_http = AAZBoolArg(
            options=['--no-http'],
            help='Disable HTTP traffic.Indicates whether HTTP traffic is not allowed on the endpoint. '
            'Default is to allow HTTP traffic.',
            blank=True
        )
        args_schema.no_https = AAZBoolArg(
            options=['--no-https'],
            help='Indicates whether HTTPS traffic is not allowed on the endpoint. '
            'Default is to allow HTTPS traffic.',
            blank=True
        )
        args_schema.enable_compression = AAZBoolArg(
            options=['--enable-compression'],
            help='If compression is enabled, content will be served as compressed '
            'if user requests for a compressed version. '
            'Content won\'t be compressed on CDN when requested content is smaller than 1 byte or larger than 1 MB.',
            blank=True
        )
        args_schema.origins._registered = False
        args_schema.is_http_allowed._registered = False
        args_schema.is_https_allowed._registered = False
        args_schema.is_compression_enabled._registered = False
        return args_schema

    def pre_operations(self):
        args = self.ctx.args

        if not 1 <= len(args.origin) <= 3 and not 5 <= len(args.origin) <= 6:
            msg = '%s takes 1, 2, 3, 5, or 6 values, %d given'
            raise argparse.ArgumentError(
                self, msg % (len(args.origin)))

        host_name = args.origin[0]
        http_port = 80
        https_port = 443
        private_link_resource_id = None
        private_link_location = None
        private_link_approval_message = None
        origin_name = host_name.to_serialized_data().replace('.', '-')

        if len(args.origin) > 1:
            http_port = int(args.origin[1].to_serialized_data())
        if len(args.origin) > 2:
            https_port = int(args.origin[2].to_serialized_data())
        if len(args.origin) > 4:
            private_link_resource_id = args.origin[3]
            private_link_location = args.origin[4]
        if len(args.origin) > 5:
            private_link_approval_message = args.origin[5]
        if len(args.origin) > 6:
            origin_name = args.origin[6]

        if http_port < 1 or http_port > 65535 or https_port < 1 or https_port > 65535:
            raise CLIError('Port number must be between 1 and 65535')

        args.origins = [{
            'name': origin_name,
            'host_name': host_name,
            'http_port': http_port,
            'https_port': https_port,
            'private_link_resource_id': private_link_resource_id,
            'private_link_location': private_link_location,
            'private_link_approval_message': private_link_approval_message
        }]

        if has_value(args.enable_compression):
            args.is_compression_enabled = args.enable_compression
        if has_value(args.no_http):
            args.is_http_allowed = not args.no_http
        if has_value(args.no_https):
            args.is_https_allowed = not args.no_https
        if args.enable_compression.to_serialized_data() and not has_value(args.content_types_to_compress):
            args.content_types_to_compress = default_content_types()


class CDNEndpointUpdate(_CDNEndpointUpdate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.no_http = AAZBoolArg(
            options=['--no-http'],
            help='Disable HTTP traffic.Indicates whether HTTP traffic is not allowed on the endpoint. '
            'Default is to allow HTTP traffic.',
            blank=True
        )
        args_schema.no_https = AAZBoolArg(
            options=['--no-https'],
            help='Indicates whether HTTPS traffic is not allowed on the endpoint. '
            'Default is to allow HTTPS traffic.',
            blank=True
        )
        args_schema.enable_compression = AAZBoolArg(
            options=['--enable-compression'],
            help='If compression is enabled, content will be served as compressed '
            'if user requests for a compressed version. '
            'Content won\'t be compressed on CDN when requested content is smaller than 1 byte or larger than 1 MB.',
            blank=True
        )
        args_schema.is_http_allowed._registered = False
        args_schema.is_https_allowed._registered = False
        args_schema.is_compression_enabled._registered = False
        args_schema.query_string_caching_behavior._default = None
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        existing = _CDNEndpointShow(cli_ctx=self.cli_ctx)(command_args={
            'resource_group': args.resource_group,
            'profile_name': args.profile_name,
            'endpoint_name': args.endpoint_name
        })
        if has_value(args.default_origin_group):
            if '/' not in args.default_origin_group.to_serialized_data():
                args.default_origin_group = f'/subscriptions/{self.ctx.subscription_id}' \
                                            f'/resourceGroups/{args.resource_group}' \
                                            f'/providers/Microsoft.Cdn/profiles/{args.profile_name}' \
                                            f'/endpoints/{args.endpoint_name}' \
                                            f'/originGroups/{args.default_origin_group}'
        if has_value(args.enable_compression):
            args.is_compression_enabled = args.enable_compression
        if not has_value(args.enable_compression):
            args.is_compression_enabled = existing['isCompressionEnabled']
        if args.is_compression_enabled.to_serialized_data() and not has_value(args.content_types_to_compress):
            args.content_types_to_compress = existing['contentTypesToCompress']
            if not has_value(args.content_types_to_compress) is None:
                args.content_types_to_compress = default_content_types()
        if has_value(args.no_http):
            args.is_http_allowed = not args.no_http
        if has_value(args.no_https):
            args.is_https_allowed = not args.no_https


class CDNEndpointRuleAdd(_CDNEndpointUpdate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.action_name = AAZStrArg(
            options=['--action-name'],
            help='The name of the action for the delivery rule: '
            'https://learn.microsoft.com/en-us/azure/cdn/cdn-standard-rules-engine-actions.',
            required=True
        )
        args_schema.order = AAZIntArg(
            options=['--order'],
            help='The order in which the rules are applied for the endpoint. Possible values {0,1,2,3,………}. '
            'A rule with a lower order will be applied before one with a higher order. '
            'Rule with order 0 is a special rule. It does not require any condition and '
            'actions listed in it will always be applied.',
            required=True
        )
        args_schema.cache_behavior = AAZStrArg(
            options=['--cache-behavior'],
            help='Caching behavior for the requests.',
            enum=['BypassCache', 'Override', 'SetIfMissing']
        )
        args_schema.cache_duration = AAZTimeArg(
            options=['--cache-duration'],
            help='The duration for which the content needs to be cached. '
            'Allowed format is hh:mm:ss.xxxxxx',
        )
        args_schema.custom_fragment = AAZStrArg(
            options=['--custom-fragment'],
            help='Fragment to add to the redirect URL.',
        )
        args_schema.custom_hostname = AAZStrArg(
            options=['--custom-hostname'],
            help='Host to redirect. Leave empty to use the incoming host as the destination host.',
        )
        args_schema.custom_path = AAZStrArg(
            options=['--custom-path'],
            help='The full path to redirect. Path cannot be empty and must start with /. '
            'Leave empty to use the incoming path as destination path.',
        )
        args_schema.custom_querystring = AAZStrArg(
            options=['--custom-querystring'],
            help='The set of query strings to be placed in the redirect URL. '
            'leave empty to preserve the incoming query string.',
        )
        args_schema.destination = AAZStrArg(
            options=['--destination'],
            help='The destination path to be used in the rewrite.'
        )
        args_schema.header_action = AAZStrArg(
            options=['--header-action'],
            help='Header action for the requests.',
            enum=['Append', 'Overwrite', 'Delete']
        )
        args_schema.header_name = AAZStrArg(
            options=['--header-name'],
            help='Name of the header to modify.',
        )
        args_schema.header_value = AAZStrArg(
            options=['--header-value'],
            help='Value of the header.',
        )
        args_schema.match_values = AAZListArg(
            options=['--match-values'],
            help='Match values of the match condition. e.g, space separated values "GET" "HTTP".',
        )
        args_schema.match_values.Element = AAZStrArg()
        args_schema.match_variable = AAZStrArg(
            options=['--match-variable'],
            help='Name of the match condition: '
            'https://learn.microsoft.com/en-us/azure/cdn/cdn-standard-rules-engine-match-conditions.',
            enum=['ClientPort', 'Cookies', 'HostName', 'HttpVersion', 'IsDevice', 'PostArgs', 'QueryString',
                  'RemoteAddress', 'RequestBody', 'RequestHeader', 'RequestMethod', 'RequestScheme', 'RequestUri',
                  'ServerPort', 'SocketAddr', 'SslProtocol', 'UrlFileExtension', 'UrlFileName', 'UrlPath']
        )
        args_schema.negate_condition = AAZBoolArg(
            options=['--negate-condition'],
            help='If true, negates the condition.',
        )
        args_schema.operator = AAZStrArg(
            options=['--operator'],
            help='Operator of the match condition.'
        )
        args_schema.preserve_unmatched_path = AAZBoolArg(
            options=['--preserve-unmatched-path'],
            help='If True, the remaining path after the source pattern will be appended to the new destination path.',
        )
        args_schema.query_parameters = AAZStrArg(
            options=['--query-parameters'],
            help='Query parameters to include or exclude (comma separated).',
        )
        args_schema.query_string_behavior = AAZStrArg(
            options=['--query-string-behavior'],
            help='Query string behavior for the requests.',
            enum=['Include', 'IncludeAll', 'Exclude', 'ExcludeAll']
        )
        args_schema.redirect_protocol = AAZStrArg(
            options=['--redirect-protocol'],
            help='Protocol to use for the redirect.',
        )
        args_schema.redirect_type = AAZStrArg(
            options=['--redirect-type'],
            help='The redirect type the rule will use when redirecting traffic.',
            enum=['Moved', 'Found', 'TemporaryRedirect', 'PermanentRedirect']
        )
        args_schema.rule_name = AAZStrArg(
            options=['--rule-name'],
            help='Name of the rule, only required for Microsoft SKU.',
        )
        args_schema.selector = AAZStrArg(
            options=['--selector'],
            help='Selector of the match condition.',
        )
        args_schema.source_pattern = AAZStrArg(
            options=['--source-pattern'],
            help='A request URI pattern that identifies the type of requests that may be rewritten.',
        )
        args_schema.transform = AAZListArg(
            options=['--transform'],
            help='Transform to apply before matching.',
        )
        args_schema.transform.Element = AAZStrArg(
            enum=['Lowercase', 'Uppercase']
        )
        args_schema.origin_group = AAZStrArg(
            options=['--origin-group'],
            help='Name of the origin group to which this rule will be added.Name or ID of the OriginGroup '
            'that would override the default OriginGroup.',
        )
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        partner_skus = [SkuName.PREMIUM_VERIZON, SkuName.CUSTOM_VERIZON, SkuName.STANDARD_VERIZON]
        profile = CDNProfileShow(cli_ctx=self.cli_ctx)(command_args={
            'resource_group': args.resource_group,
            'profile_name': args.profile_name
        })
        if not has_value(args.rule_name) and profile['sku']['name'] not in partner_skus:
            raise CLIError("--rule-name is required for Microsoft SKU")
        endpoint = _CDNEndpointShow(cli_ctx=self.cli_ctx)(command_args={
            'resource_group': args.resource_group,
            'profile_name': args.profile_name,
            'endpoint_name': args.endpoint_name
        })

        delivery_policy = None
        if 'deliveryPolicy' in endpoint:
            delivery_policy = create_delivery_policy_from_existing(endpoint['deliveryPolicy'])
        else:
            delivery_policy = {
                'description': 'default_policy',
                'rules': []
            }

        conditions = []
        condition = create_condition(args.match_variable, args.operator, args.match_values,
                                     args.selector, args.negate_condition, args.transform)
        if condition is not None:
            conditions.append(condition)
        actions = []
        action = create_action(args.action_name, args.cache_behavior, args.cache_duration, args.header_action,
                               args.header_name, args.header_value, args.query_string_behavior, args.query_parameters,
                               args.redirect_type, args.redirect_protocol, args.custom_hostname, args.custom_path,
                               args.custom_querystring, args.custom_fragment, args.source_pattern, args.destination,
                               args.preserve_unmatched_path, self.ctx.subscription_id,
                               args.resource_group, args.profile_name, args.endpoint_name, args.origin_group)
        if action is not None:
            actions.append(action)
        rule = {
            'name': args.rule_name,
            'order': args.order,
            'conditions': conditions,
            'actions': actions
        }
        delivery_policy['rules'].append(rule)
        args.delivery_policy = delivery_policy


class CDNEndpointRuleRemove(_CDNEndpointUpdate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.rule_name = AAZStrArg(
            options=['--rule-name'],
            help='Name of the rule.',
        )
        args_schema.order = AAZIntArg(
            options=['--order'],
            help='The order in which the rules are applied for the endpoint. Possible values {0,1,2,3,………}. '
            'A rule with a lower order will be applied before one with a higher order. '
            'Rule with order 0 is a special rule. It does not require any condition and '
            'actions listed in it will always be applied.',
        )
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        if not has_value(args.rule_name) and not has_value(args.order):
            raise CLIError("Either --rule-name or --order must be specified")

        if has_value(args.order) and args.order < 0:
            raise CLIError("Order should be non-negative.")
        endpoint = _CDNEndpointShow(cli_ctx=self.cli_ctx)(command_args={
            'resource_group': args.resource_group,
            'profile_name': args.profile_name,
            'endpoint_name': args.endpoint_name
        })
        delivery_policy = create_delivery_policy_from_existing(endpoint['deliveryPolicy'])
        if delivery_policy is not None:
            pop_index = -1
            for idx, rule in enumerate(delivery_policy['rules']):
                if has_value(args.rule_name) and rule['name'] == args.rule_name:
                    pop_index = idx
                    break
                if args.order is not None and rule['order'] == args.order:
                    pop_index = idx
                    break

            # To guarantee the consecutive rule order,
            # we need to make sure the rule with order larger than the deleted one
            # to decrease its order by one. Rule with order 0 is special and no rule order adjustment is required.
            if pop_index != -1:
                pop_order = delivery_policy['rules'][pop_index]['order']
                delivery_policy['rules'].pop(pop_index)
                for rule in delivery_policy['rules']:
                    if rule['order'] > pop_order and pop_order != 0:
                        rule['order'] -= 1

        else:
            logger.warning("rule cannot be found. This command will be skipped. Please check the rule name")
        args.delivery_policy = delivery_policy


class CDNEndpointRuleActionAdd(_CDNEndpointUpdate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.action_name = AAZStrArg(
            options=['--action-name'],
            help='The name of the action for the delivery rule: '
            'https://learn.microsoft.com/en-us/azure/cdn/cdn-standard-rules-engine-actions.',
            required=True
        )
        args_schema.cache_behavior = AAZStrArg(
            options=['--cache-behavior'],
            help='Caching behavior for the requests.',
            enum=['BypassCache', 'Override', 'SetIfMissing']
        )
        args_schema.cache_duration = AAZTimeArg(
            options=['--cache-duration'],
            help='The duration for which the content needs to be cached. '
            'Allowed format is hh:mm:ss.xxxxxx',
        )
        args_schema.custom_fragment = AAZStrArg(
            options=['--custom-fragment'],
            help='Fragment to add to the redirect URL.',
        )
        args_schema.custom_hostname = AAZStrArg(
            options=['--custom-hostname'],
            help='Host to redirect. Leave empty to use the incoming host as the destination host.',
        )
        args_schema.custom_path = AAZStrArg(
            options=['--custom-path'],
            help='The full path to redirect. Path cannot be empty and must start with /. '
            'Leave empty to use the incoming path as destination path.',
        )
        args_schema.custom_querystring = AAZStrArg(
            options=['--custom-querystring'],
            help='The set of query strings to be placed in the redirect URL. '
            'leave empty to preserve the incoming query string.',
        )
        args_schema.destination = AAZStrArg(
            options=['--destination'],
            help='The destination path to be used in the rewrite.'
        )
        args_schema.header_action = AAZStrArg(
            options=['--header-action'],
            help='Header action for the requests.',
            enum=['Append', 'Overwrite', 'Delete']
        )
        args_schema.header_name = AAZStrArg(
            options=['--header-name'],
            help='Name of the header to modify.',
        )
        args_schema.header_value = AAZStrArg(
            options=['--header-value'],
            help='Value of the header.',
        )
        args_schema.preserve_unmatched_path = AAZBoolArg(
            options=['--preserve-unmatched-path'],
            help='If True, the remaining path after the source pattern will be appended to the new destination path.',
        )
        args_schema.query_parameters = AAZStrArg(
            options=['--query-parameters'],
            help='Query parameters to include or exclude (comma separated).',
        )
        args_schema.query_string_behavior = AAZStrArg(
            options=['--query-string-behavior'],
            help='Query string behavior for the requests.',
            enum=['Include', 'IncludeAll', 'Exclude', 'ExcludeAll']
        )
        args_schema.redirect_protocol = AAZStrArg(
            options=['--redirect-protocol'],
            help='Protocol to use for the redirect.',
        )
        args_schema.redirect_type = AAZStrArg(
            options=['--redirect-type'],
            help='The redirect type the rule will use when redirecting traffic.',
            enum=['Moved', 'Found', 'TemporaryRedirect', 'PermanentRedirect']
        )
        args_schema.rule_name = AAZStrArg(
            options=['--rule-name'],
            help='Name of the rule, only required for Microsoft SKU.',
            required=True,
        )
        args_schema.source_pattern = AAZStrArg(
            options=['--source-pattern'],
            help='A request URI pattern that identifies the type of requests that may be rewritten.',
        )
        args_schema.origin_group = AAZStrArg(
            options=['--origin-group'],
            help='Name of the origin group to which this rule will be added.Name or ID of the OriginGroup '
            'that would override the default OriginGroup.',
        )
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        endpoint = _CDNEndpointShow(cli_ctx=self.cli_ctx)(command_args={
            'resource_group': args.resource_group,
            'profile_name': args.profile_name,
            'endpoint_name': args.endpoint_name
        })

        delivery_policy = create_delivery_policy_from_existing(endpoint['deliveryPolicy'])
        action = create_action(args.action_name, args.cache_behavior, args.cache_duration, args.header_action,
                               args.header_name, args.header_value, args.query_string_behavior, args.query_parameters,
                               args.redirect_type, args.redirect_protocol, args.custom_hostname, args.custom_path,
                               args.custom_querystring, args.custom_fragment, args.source_pattern, args.destination,
                               args.preserve_unmatched_path, self.ctx.subscription_id,
                               args.resource_group, args.profile_name, args.endpoint_name, args.origin_group)
        for i in range(0, len(delivery_policy['rules'])):
            if delivery_policy['rules'][i]['name'] == args.rule_name:
                delivery_policy['rules'][i]['actions'].append(action)
        args.delivery_policy = delivery_policy


class CDNEndpointRuleActionRemove(_CDNEndpointUpdate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.index = AAZIntArg(
            options=['--index'],
            help='The index of the condition/action.',
            required=True
        )
        args_schema.rule_name = AAZStrArg(
            options=['--rule-name'],
            help='Name of the rule.',
            required=True,
        )
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        endpoint = _CDNEndpointShow(cli_ctx=self.cli_ctx)(command_args={
            'resource_group': args.resource_group,
            'profile_name': args.profile_name,
            'endpoint_name': args.endpoint_name
        })

        delivery_policy = create_delivery_policy_from_existing(endpoint['deliveryPolicy'])
        if delivery_policy is not None:
            for i in range(0, len(delivery_policy['rules'])):
                if delivery_policy['rules'][i]['name'] == args.rule_name:
                    delivery_policy['rules'][i]['actions'].pop(args.index.to_serialized_data())
        else:
            logger.warning("rule cannot be found. This command will be skipped. Please check the rule name")
        args.delivery_policy = delivery_policy


class CDNEndpointRuleConditionAdd(_CDNEndpointUpdate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.match_values = AAZListArg(
            options=['--match-values'],
            help='Match values of the match condition. e.g, space separated values "GET" "HTTP".',
        )
        args_schema.match_values.Element = AAZStrArg()
        args_schema.match_variable = AAZStrArg(
            options=['--match-variable'],
            help='Name of the match condition: '
            'https://learn.microsoft.com/en-us/azure/cdn/cdn-standard-rules-engine-match-conditions.',
            enum=['ClientPort', 'Cookies', 'HostName', 'HttpVersion', 'IsDevice', 'PostArgs', 'QueryString',
                  'RemoteAddress', 'RequestBody', 'RequestHeader', 'RequestMethod', 'RequestScheme', 'RequestUri',
                  'ServerPort', 'SocketAddr', 'SslProtocol', 'UrlFileExtension', 'UrlFileName', 'UrlPath'],
            required=True
        )
        args_schema.negate_condition = AAZBoolArg(
            options=['--negate-condition'],
            help='If true, negates the condition.',
        )
        args_schema.operator = AAZStrArg(
            options=['--operator'],
            help='Operator of the match condition.',
            required=True
        )
        args_schema.rule_name = AAZStrArg(
            options=['--rule-name'],
            help='Name of the rule, only required for Microsoft SKU.',
            required=True,
        )
        args_schema.selector = AAZStrArg(
            options=['--selector'],
            help='Selector of the match condition.',
        )
        args_schema.transform = AAZListArg(
            options=['--transform'],
            help='Transform to apply before matching.',
        )
        args_schema.transform.Element = AAZStrArg(
            enum=['Lowercase', 'Uppercase']
        )
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        endpoint = _CDNEndpointShow(cli_ctx=self.cli_ctx)(command_args={
            'resource_group': args.resource_group,
            'profile_name': args.profile_name,
            'endpoint_name': args.endpoint_name
        })

        delivery_policy = create_delivery_policy_from_existing(endpoint['deliveryPolicy'])
        condition = create_condition(args.match_variable, args.operator, args.match_values,
                                     args.selector, args.negate_condition, args.transform)
        for i in range(0, len(delivery_policy['rules'])):
            if delivery_policy['rules'][i]['name'] == args.rule_name:
                delivery_policy['rules'][i]['conditions'].append(condition)

        args.delivery_policy = delivery_policy


class CDNEndpointRuleConditionRemove(_CDNEndpointUpdate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.index = AAZIntArg(
            options=['--index'],
            help='The index of the condition/action.',
            required=True
        )
        args_schema.rule_name = AAZStrArg(
            options=['--rule-name'],
            help='Name of the rule.',
            required=True,
        )
        return args_schema

    def pre_operations(self):
        args = self.ctx.args
        endpoint = _CDNEndpointShow(cli_ctx=self.cli_ctx)(command_args={
            'resource_group': args.resource_group,
            'profile_name': args.profile_name,
            'endpoint_name': args.endpoint_name
        })

        delivery_policy = create_delivery_policy_from_existing(endpoint['deliveryPolicy'])
        if delivery_policy is not None:
            for i in range(0, len(delivery_policy['rules'])):
                if delivery_policy['rules'][i]['name'] == args.rule_name:
                    delivery_policy['rules'][i]['conditions'].pop(args.index.to_serialized_data())
        else:
            logger.warning("rule cannot be found. This command will be skipped. Please check the rule name")
        args.delivery_policy = delivery_policy


class CdnMigrateToAfd(_Migrate):
    @classmethod
    def _build_arguments_schema(cls, *args, **kwargs):
        args_schema = super()._build_arguments_schema(*args, **kwargs)
        args_schema.identity_type = AAZStrArg(
            options=['--identity-type'],
            help='Type of managed service identity (where both SystemAssigned and UserAssigned types are allowed).',
            enum=['SystemAssigned', 'None', 'UserAssigned', 'SystemAssigned, UserAssigned'],
        )
        args_schema.user_assigned_identities = AAZListArg(
            options=['--user-assigned-identities'],
            help='The set of user assigned identities associated with the resource. '
            'The userAssignedIdentities dictionary keys will be ARM resource ids in the form: '
            '\'/subscriptions/{{subscriptionId}}/resourceGroups/{{resourceGroupName}}'
            '/providers/Microsoft.ManagedIdentity/userAssignedIdentities/{{identityName}}. '
            'The dictionary values can be empty objects ({{}}) in requests.',
        )
        args_schema.user_assigned_identities.Element = AAZStrArg()
        return args_schema

    def post_operations(self):
        args = self.ctx.args
        identity = None
        user_assigned_identities = {}
        for identity in args.user_assigned_identities:
            user_assigned_identities[identity.to_serialized_data()] = {}
        if args.identity_type == 'UserAssigned' or args.identity_type == 'SystemAssigned, UserAssigned':
            identity = {
                'type': args.identity_type,
                'userAssignedIdentities': user_assigned_identities
            }
        elif args.identity_type == 'SystemAssigned':
            identity = {
                'type': args.identity_type
            }

        existing = _AFDProfileShow(cli_ctx=self.cli_ctx)(command_args={
            'resource_group': args.resource_group,
            'profile_name': args.profile_name
        })
        location = None if 'location' not in existing else existing['location']
        _AFDProfileUpdate(cli_ctx=self.cli_ctx)(command_args={
            'resource_group': args.resource_group,
            'profile_name': args.profile_name,
            'location': location,
            'identity': identity
        })
